Ext.namespace("dSS", "dSS.config", "dSS.devprops", "dSS.gthelper");

dSS.devprops.CreatePanelForDeviceProperties = function(record, featureConfig,
                                                      scope) {
    var devInfo = dSS.util.parseProductID(record.get('productID'));
    var type = devInfo[0];
    var deviceNumber = devInfo[1];
    var cls = dSS.util.parseFunctionID(record.get('functionID'));
    var stdForms = [];
    var advancedForms = [];
    var loaders = [];
    var blinkForms = [];
    var transitionForms = [];
    var showDeviceInfo = true;
    var matrix;
    var extraTitle;
    var width = 400;

    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        var addpboutmodeloader = false;
        var addLocalPrioLoader = false;
        var color = cls[0];

        if (matrix.enabletypehack === true) {
            stdForms.push(
                {
                    xtype: 'dssdevinputtypeform',
                    dSUID: record.get('dsuid'),
                    matrix: matrix,
                    color: color
                });
            addpboutmodeloader = true;
        }

        if ((matrix.jokerconfig === true) &&
            (cls[0] === dSS.config.deviceClass.SW)) {
            color = dSS.util.getJokerGroup(record.get('groups'));
            // assume it's not yet configured
            if (color === null) {
                color = dSS.config.deviceClass.SW;
            }
            stdForms.push(
                {
                    xtype: 'dssdevjokerconfigform',
                    dSUID: record.get('dsuid'),
                    device: record,
                    product: devInfo[0],
                    group: color,
                    matrix: matrix,
                    hidden: (matrix.enabletypehack === true)
                });
        }

        if (matrix.heatinggroup === true) {
            var arr = record.get('groups');
            var grp = dSS.config.groups.HEATING;
            // in reality the length of this array should always be 1
            if (arr.length > 0) {
                grp = arr[0];
            }
            stdForms.push(
                {
                    xtype: 'dssdevheatingconfigform',
                    dSUID: record.get('dsuid'),
                    device: record,
                    product: devInfo[0],
                    group: grp
                });
        }

        if (matrix.valvetype === true) {
            stdForms.push(
                {
                    xtype: 'dssdevvalvetypeform',
                    device: record,
                    matrix: matrix
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.VALVE_TYPE,
                         url: '/json/device/getValveType',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.optypeconfig === true) {
            stdForms.push(
                {
                    xtype: 'dssdevoptypeform',
                    dSUID: record.get('dsuid')
                });
            addpboutmodeloader = true;
        }

        var override = color;
        if (matrix.heatinggroup === true) {
            var arr = record.get('groups');
            // in reality the length of this array should always be 1
            if (arr.length > 0) {
                override = arr[0];
            }
        }

        if ((matrix.pushbutton === true) &&
            (record.get('buttonInputCount') > 0) &&
            ((record.get('productRevision') >= dSS.config.PushbuttonMinFW) ||
             (record.get('isVdcDevice') === true))) {
            stdForms.push(
                {
                    xtype: 'dssdevgrouppushbuttonform',
                    dSUID : record.get('dsuid'),
                    product: type,
                    productNumber: deviceNumber,
                    matrix: matrix,
                    group: override,
                    hidden: (matrix.enabletypehack === true)

                });

            if (!((cls[0] === dSS.config.deviceClass.BL) &&
                  (type === dSS.config.deviceTypeNum.KM) &&
                  (deviceNumber === 200))) {
                stdForms.push(
                    {
                        xtype: 'dssdevbuttonusageform',
                        device: record
                    });

                loaders.push(new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.BUTTON_USAGE,
                    url: '/json/device/getButtonUsage',
                    params:
                    {
                        dsuid: record.get('dsuid')
                    }
                }));
            }

            if (matrix.pushbadvanced === true) {
                advancedForms.push(
                    {
                        xtype: 'dssdevgrouppushbuttonadvancedform',
                        device : record,
                        matrix: matrix,
                        product: devInfo[0],
                        productNumber: deviceNumber,
                        group: override
                     });
            } else {
                stdForms.push(
                {
                    xtype: 'dssdevgrouppushbuttonadvancedform',
                    device : record,
                    matrix: matrix,
                    product: devInfo[0],
                    productNumber: deviceNumber,
                    group: override,
                    disabled: true,
                    hidden: true
                });
            }

            addpboutmodeloader = true;
            addLocalPrioLoader = true;
        }

        var renderMultibuttonConfigForm = false;
        if ((matrix.pushbutton === true) &&
            (matrix.twowayconfig === true) &&
            (dSS.util.deviceHasMultipleButtons(record.get('functionID')) === true)) {
            if ((record.get('buttonInputIndex') === 0) ||
                (record.get('buttonInputIndex') === 2)) {
                renderMultibuttonConfigForm = true;
            }
        }

        if (type === dSS.config.deviceTypeNum.UMR) {
            if (record.get('dSUIDIndex') > 0) {
                renderMultibuttonConfigForm = false;
            } else {
                renderMultibuttonConfigForm = true;
            }
        }

        if (renderMultibuttonConfigForm === true) {
            stdForms.push(
                new dSS.devprops.MultiButtonConfigForm(
                {
                    record: record,
                    group: color,
                    matrix: matrix,
                    hidden: (matrix.enabletypehack === true)
                }));
        }

        if (matrix.inputmode === true) {
            stdForms.push(
                {
                    xtype: 'dssdevinputmodeform',
                    dSUID: record.get('dsuid'),
                    matrix: matrix
                });
            addpboutmodeloader = true;
        }

        if (matrix.akmsensor === true) {
            stdForms.push(
                new dSS.devprops.AKMSensorConfigForm(
                {
                    device: record,
                    group: color,
                    matrix: matrix
                }));

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_BINARY_INPUTS,
                         url: '/json/device/getBinaryInputs',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.akminput === true) {
            stdForms.push(
                {
                    xtype: 'dssakminputconfigform',
                    device: record,
                    matrix: matrix
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_INPUT_PROPERTY,
                         url: '/json/property/getString',
                         params:
                         {
                             path: '/apartment/zones/zone0/devices/' +
                                   record.get('dsuid') +
                                   '/AKMInputProperty'
                         }
                     }));
        }

        if (matrix.akmdelay === true) {
            stdForms.push(
                {
                    xtype: 'dssakmdelayconfigform',
                    device: record,
                    group: color,
                    matrix: matrix
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_DELAY_CONFIG,
                         url: '/json/device/getAKMInputTimeouts',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if ((matrix.outmode === true) || (matrix.outmodeswitch === true) 
            || (matrix.umroutmode === true)
            || (matrix.outmodegeneric == true) 
            || (matrix.outmodeenoceanvalve == true)
            || ((matrix.heatingoutmode === true) && (record.get('VdcHardwareModelGuid') === ''))
            || (matrix.extradimmer === true)) {
            stdForms.push(
                {
                    xtype: 'dssdevoutputmodeform',
                    device: record,
                    group: override,
                    matrix: matrix
                });
            addpboutmodeloader = true;

            if (matrix.outmodeauto === true) {
                loaders.push(new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CURRENT_OUTPUT_AUTO_MODE,
                    url: '/json/device/getOutputValue',
                    params:
                    {
                        dsuid: record.get('dsuid'),
                        type: 'currentOutputMode'
                    }
                }));
            }
        }

        if (matrix.umvrelay === true) {
            stdForms.push(
                {
                    xtype: 'dssdevumvrelayconfigform',
                    device: record
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.UMV_RELAY,
                         url: '/json/device/getUMVRelayValue',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.ftwtempcontrolventilationselect === true) {
            stdForms.push(
                {
                    xtype: 'dssdevftwtempcontrolconfig',
                    device: record
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.FTW_MODE,
                         url: '/json/device/getSK204Config',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.ventconfig === true) {
            // hack in order to be able to comply with the new spec
            var usedSUID = record.get('dsuid');
            var usePresent = record.get('isPresent');
            var useGroup = dSS.util.getJokerGroup(record.get('groups'));

            if (record.get('dSUIDIndex') == 0) {
                var paired = record.get('pairedDevices');
                if (paired.length > 0) {
                    usedSUID = paired[0];
                    var store = dSS.data.Loader.getDeviceStore();
                    var index = store.find('dsuid', paired[0]);
                    if (index != -1) {
                        var devrec = store.getAt(index);
                        usePresent = devrec.get('isPresent');
                        useGroup = dSS.util.getJokerGroup(devrec.get('groups'));
                    } else {
                        usePresent = false;
                    }
                }
            }

            stdForms.push(
                {
                    xtype: 'dssdevventilationconfig',
                    dSUID: usedSUID,
                    present: usePresent,
                    group: useGroup
                });
        }

        if (matrix.ftwdisplaysettings === true) {
            stdForms.push(
                {
                    xtype: 'dssdevftwdisplaysettings',
                    device: record
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.FTW_DISPLAY_SETTINGS,
                         url: '/json/device/getSK204DisplayMode',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.ftwbacklighttimeout === true) {
            stdForms.push(
                {
                    xtype: 'dssdevftwbacklighttimeout',
                    device: record
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.FTW_BACKLIGHT_TIMEOUT,
                         url: '/json/device/getSK204BacklightTimeout',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.temperatureoffset === true) {
            var sensorIndex = dSS.util.GetSensorIndex(record,
                                    dSS.config.SensorTypes.TEMPERATURE_INDOORS);

            if (sensorIndex != -1) {
                stdForms.push(
                    {
                        xtype: 'temperatureoffsetconfigform',
                        device: record
                    });

                loaders.push(new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SENSOR_VALUE,
                    url: '/json/device/getSensorValue',
                    params:
                    {
                        dsuid: record.get('dsuid'),
                        sensorIndex: sensorIndex
                    }
                }));

                loaders.push(new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.TEMPERATURE_OFFSET,
                    url: '/json/device/getTemperatureOffset',
                    params:
                    {
                        dsuid: record.get('dsuid')
                    }
                }));
            }
        }

        if (addpboutmodeloader === true) {
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.COMBINED_PB_OUTMODE,
                         url: '/json/property/query',
                         params:
                         {
                            query: '/apartment/zones/zone0/devices/' +
                                 record.get('dsuid') +
                                 '(outputMode)/button(id,inputMode,setsLocalPriority,callsPresent,activeGroup)'
                         }
                     }));
        }

        if (addLocalPrioLoader === true) {
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.SCENE_CONFIGURATION,
                         url: '/json/device/getSceneMode',
                         params:
                         {
                             sceneID : dSS.config.scenes.LOCAL_ON,
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.blinkconfig === true) {
            blinkForms.push(
                {
                    xtype: 'outputblinkconfigform',
                    device: record,
                    matrix: matrix
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.BLINK_CONFIG,
                         url: '/json/device/getBlinkConfig',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.impulseconfig === true) {
            blinkForms.push(
                {
                    xtype: 'dssdevoutputimpulseconfigform',
                    device: record,
                    matrix: matrix
                });
            loaders.push(new dSS.devprops.Loader(
            {
                loaderId: dSS.devprops.Loaders.OUTPUT_AFTER_IMPULSE,
                url: '/json/device/getOutputAfterImpulse',
                params:
                {
                   dsuid: record.get('dsuid')
                }
            }));
        }

        if (matrix.dimtimeconfig === true) {
            transitionForms.push(
            {
                xtype: 'dssramptimeconfigform',
                device: record
            });

            loaders.push(new dSS.devprops.Loader(
            {
                loaderId: dSS.devprops.Loaders.DIMTIME0,
                url: '/json/device/getTransitionTime',
                params:
                {
                   dsuid: record.get('dsuid'),
                   dimtimeIndex: 0
                }
            }));

            loaders.push(new dSS.devprops.Loader(
            {
                loaderId: dSS.devprops.Loaders.DIMTIME2,
                url: '/json/device/getTransitionTime',
                params:
                {
                   dsuid: record.get('dsuid'),
                   dimtimeIndex: 2
                }
            }));
        }

        if (matrix.locationconfig) {
            stdForms.push(
                {
                    xtype: 'dssdevlocationform',
                    device: record
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.LOCATION,
                         url: '/json/device/getCardinalDirection',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));

        }

        if (matrix.windprotectionconfigawning || matrix.windprotectionconfigblind) {
            stdForms.push(
                {
                    xtype: 'dssdevwindprotectionclassform',
                    device: record,
                    matrix: matrix
                });

            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.WIND_PROTECTION,
                         url: '/json/device/getWindProtectionClass',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.outconfigswitch === true) {
            advancedForms.push(
                {
                    xtype: 'outputswitchtresholdconfigform',
                    device: record
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.OUTPUT_SWITCH_THRESHOLD,
                         url: '/json/device/getSwitchThreshold',
                         params:
                         {
                             dsuid: record.get('dsuid')
                         }
                     }));
        }

        if (matrix.operationlock === true) {
            advancedForms.push(
                {
                    xtype: 'operationlockignoreconfigform',
                    device: record
                });
        }
    } // CIRCUITS

    var jumpMode = false;
    var additionalInfo = [];
    var tabHeight;
    var devDsuid;
    var devDsid;

    var reason = dSS._("Modifying settings for this device is not allowed!");
    var confLock = record.get('configurationLocked');
    var oemState = record.get('OemStatus');
    if (confLock && ((oemState === 'Unknown') || (oemState === 'Loading'))) {
        reason = dSS._("Modifying settings for this device is not allowed, because device configuration has not been read out yet!");
    }

    var customTitle;
    if (featureConfig === dSS.config.FeatureConfig.SCENE) {
        customTitle = dSS._("Activity settings");
        tabHeight = 170;
        var scene;
        jumpMode = true;
        additionalInfo.push(
        {
            key: dSS._("Output mode"),
            value: dSS._(dSS.config.OutModeStrings[record.get('outputMode')])
        });
        try {
            scene = scope.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var sceneId = record.get('zoneId') + '-' + cls[0] + '-' + scene;
        var sceneRec = sceneStore.getById(sceneId);
        var sceneName = "";
        if (sceneRec !== undefined) {
            sceneName = sceneRec.get('name');
        }

        if (sceneName.length > 0) {
            extraTitle = ' - ' + sceneName;
        } else {
            extraTitle = ' - ' + dSS._(dSS.config.sceneNames[scene]);
        }

        if (dSS.devprops.CheckShowSceneProperties(matrix) === true) {
            stdForms.push({
                        xtype: 'dssdevscenepropsform',
                        deviceRecord: record,
                        scene: scene,
                        visibilityMatrix: matrix
                   });
            advancedForms.push({
                        xtype: 'dssdevscenepropsadvancedform',
                        deviceRecord: record,
                        scene: scene,
                        visibilityMatrix: matrix
                   });
            if (record.get('isPresent') === true) {
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.SCENE_MODE,
                             url: '/json/device/getSceneMode',
                             params:
                             {
                                sceneID : scene,
                                dsuid: record.get('dsuid')
                             }
                         }));
            }
        }

        if ((scene >= dSS.config.scenes.PRESENT) ||
            (scene == dSS.config.scenes.PANIC)) {
            tabHeight = 415;
            stdForms.push({
                    xtype: 'dssdevicesettingswrapper',
                    deviceRecord: record,
                    scene: scene,
                    visibilityMatrix: matrix
            });

            if (record.get('isPresent') === true) {
                if (matrix.outputchannels) {
                    loaders.push(new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS,
                        url: '/json/device/getOutputChannelSceneValue',
                        params:
                        {
                            sceneNumber: scene,
                            channels: 'hue;saturation;brightness',
                            dsuid: record.get('dsuid')
                        }
                    }));
                } else if (matrix.fcu) {
                    loaders.push(new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS,
                        url: '/json/device/getOutputChannelSceneValue2',
                        params:
                        {
                            sceneNumber: scene,
                            channels: 'airFlowIntensity;airLouverPosition',
                            dsuid: record.get('dsuid')
                        }
                    }));
                } else if (matrix.outvalue8 || matrix.shadeposition) {
                    loaders.push(new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.SCENE_VALUE,
                        url: '/json/device/getSceneValue',
                        params:
                        {
                            sceneID: scene,
                            dsuid: record.get('dsuid')
                        }
                    }));
                }
            }
        }

        // check if device is locked in an apartment scene
        if (record.get('clusterLock') === true) {
            var lockedScenes = record.get('lockedScenes');
            if (lockedScenes.indexOf(scene) !== -1) {
                confLock = true;
            }
        }

        // otherwise we could be in some zone scene which contains a
        // locked device (not necessarily ours)
        if (!confLock) {
            var zoneID = record.get('zoneId');
            if (zoneID != 0) {
                var zrecord = dSS.data.Loader.getZoneStore().getById(zoneID);
                if (zrecord) {
                    if (!dSS.util.isApartmentScene(scene)) {
                        var locks = zrecord.get('clusterLocks');
                        Ext.each(locks, function(lock) {
                            if ((lock.deviceClass == cls[0]) ||
                                ((cls[0] === dSS.config.deviceClass.BL) &&
                                 (lock.deviceClass === dSS.util.getHeatingGroup(record.get('groups')))) ||
                                ((cls[0] === dSS.config.deviceClass.SW) &&
                                 (lock.deviceClass === dSS.util.getJokerGroup(record.get('groups'))))) {
                                if (lock.lockedScenes.indexOf(scene) !== -1) {
                                    confLock = true;
                                }
                            }
                        }); // each
                    } else {
                        var locks = record.get('lockedApartmentScenes');
                        if (locks.indexOf(scene) !== -1) {
                            confLock = true;
                        }
                    }
                } // if zrecord
            } // zone != 0
        }
    } else {
        devDsuid = record.get('dsuid');
        devDsid = record.get('dsid');
    }

    var forms = [];

    if (stdForms.length > 0) {
        forms.push(
            {
                title: dSS._("Settings"),
                forms: stdForms
            }
        );
    }

    if (advancedForms.length > 0) {
        forms.push(
            {
                title: dSS._("Advanced settings"),
                forms: advancedForms
            }
        );
    }

    if (blinkForms.length > 0) {
        var tabtitle = dSS._("Blink");
        if (type === dSS.config.deviceTypeNum.UMR) {
            tabtitle = dSS._("Impulse");
        }
        forms.push(
            {
                title: tabtitle,
                forms: blinkForms
            }
        );
    }

    if (transitionForms.length > 0) {
        forms.push(
            {
                title: dSS._("Transition"),
                forms: transitionForms
            }
        );
        width = 450;
    }
    var retObj;
    
    if ((advancedForms.length == 0) && (blinkForms.length == 0)) {
        retObj =
        {
            panel:  new dSS.devprops.DevicePropertiesPanel({
                        dSUID: devDsuid,
                        dSID: devDsid,
                        DisplayID: record.get('DisplayID'),
                        name: record.get('name'),
                        showDefaults: true,
                        additionalInfo: additionalInfo,
                        standardForms: stdForms,
                        loaders: loaders,
                        tabHeight: tabHeight,
                        stayMasked: confLock,
                        maskReason: reason
                    }),
            title: customTitle,
            extraTitle: extraTitle,
            windowWidth: width
        };
    } else {
        retObj =
        {
            panel:  new dSS.devprops.DevicePropertiesMultitabPanel({
                        dSUID: devDsuid,
                        dSID: devDsid,
                        DisplayID: record.get('DisplayID'),
                        name: record.get('name'),
                        showDefaults: true,
                        additionalInfo: additionalInfo,
                        tabForms: forms,
                        loaders: loaders,
                        tabHeight: tabHeight,
                        stayMasked: confLock,
                        maskReason: reason
                    }),
            title: customTitle,
            extraTitle: extraTitle,
            windowWidth: width
        };
    }
    return retObj;
};

dSS.devprops.CreatePanelForShadeProperties = function(record) {
    var matrix;
    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (matrix.shadeprops !== true) {
        Ext.Msg.alert(dSS._("Error"), dSS._("Shade properties can only be configured for grey klemme L devices"));
        return;
    }

    var shadeloaders =
        [
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_OPEN_CLOSE,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsuid: record.get('dsuid'),
                    'class': 3,
                    index: 0x32
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_MTIME_BLINK,
                url: '/json/device/getConfig',
                params:
                {
                    dsuid: record.get('dsuid'),
                    'class': 3,
                    index: 0x11
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_CLOSE_OPEN,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsuid: record.get('dsuid'),
                    'class': 3,
                    index: 0x30
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsuid: record.get('dsuid'),
                    'class': 3,
                    index: 0x38
                }
            })
        ];

    if (matrix.motiontimefins === true) {
        shadeloaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_MTIME_FINS,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsuid: record.get('dsuid'),
                    'class': 3,
                    index: 0x3e
                }
            })
        );
    }

    shadeloaders = shadeloaders.concat(
         [
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE,
                 url: '/json/device/getConfigWord',
                 params:
                 {
                     dsuid: record.get('dsuid'),
                     'class': 3,
                     index: 0x3a
                 }
             }),
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_CALIBRATION_ALLOWED,
                 url: '/json/device/getConfig',
                 params:
                 {
                     dsuid: record.get('dsuid'),
                     'class': 3,
                     index: 0x3d
                 }
             }),
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH,
                 url: '/json/device/getConfigWord',
                 params:
                 {
                     dsuid: record.get('dsuid'),
                     'class': 3,
                     index: 0x36
                 }
             }),
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_POS_TIME_MAX,
                 url: '/json/device/getMaxMotionTime',
                 params:
                 {
                     dsuid: record.get('dsuid')
                 }
             })
         ]
    );

    var standardForms =
    [
        {
            xtype: 'shadepropsform',
            dSUID: record.get('dsuid'),
            matrix: matrix
        }
    ];

    var advancedForms =
    [
        {
            xtype: 'shadepropsadvancedform',
            dSUID: record.get('dsuid')
        }
    ];

    var forms =
    [
        {
            title: dSS._("Settings"),
            forms: standardForms
        },
        {
            title: dSS._("Advanced settings"),
            forms: advancedForms
        }
    ];

    var retObj =
    {
        panel:    new dSS.devprops.DevicePropertiesMultitabPanel({
                        DisplayID: record.get('DisplayID'),
                        name: record.get('name'),
                        showDefaults: true,
                        tabForms: forms,
                        loaders: shadeloaders
                    }),
        extraTitle: dSS._("shade")
    };

    return retObj;
};

dSS.devprops.CreatePanelForHeatingProperties = function(record) {
    var matrix;
    try {
        matrix = dSS.devprops.GetVisibleProperties(record);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (matrix.heatingprops !== true) {
        Ext.Msg.alert(dSS._("Error"), dSS._("Climate properties can only be configured for blue klemme M devices"));
        return;
    }

    var standardForms = [];
    var forms = [];
    var heatingloaders = [];

    // hack for the UMR with new firmware
    if (matrix.umroutmodeswitch2x !== true) {
        heatingloaders.push(
           new dSS.devprops.Loader({
               loaderId: dSS.devprops.Loaders.VALVE_CONTROL_MODE,
               url: '/json/device/getValveControlMode',
               params:
               {
                   dsuid: record.get('dsuid'),
               }
           })
           );
    }

    var outputMode = record.get('outputMode');
    switch (outputMode)
    {
        case dSS.config.deviceParams.OUTPUT_MODE_PWM:
        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM:
            heatingloaders.push(
               new dSS.devprops.Loader({
                   loaderId: dSS.devprops.Loaders.VALVE_TIMER_MODE,
                   url: '/json/device/getValveTimerMode',
                   params:
                   {
                       dsuid: record.get('dsuid'),
                   }
               })
            );
        // fall through since PWM control mode needs both loaders
        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS:
        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_PARALLEL:
            heatingloaders.push(
               new dSS.devprops.Loader({
                   loaderId: dSS.devprops.Loaders.VALVE_PWM_MODE,
                   url: '/json/device/getValvePwmMode',
                   params:
                   {
                       dsuid: record.get('dsuid'),
                   }
               })
            );
            break;
        default:
            break;
    }

    if (matrix.umroutmodeswitch2x !== true) {
        standardForms =
        [
            {
                xtype: 'heatingpropsform',
                dSUID: record.get('dsuid'),
                outputMode: record.get('outputMode')
            }
        ];

        forms.push(
        {
            title: dSS._("Settings"),
            forms: standardForms
        });
    }

    var th = 350;
    if (dSS.util.isHeatingPWMMode(record.get('outputMode'))) {
        th = 510;
    }

    if (dSS.util.isHeatingPWMMode(record.get('outputMode')) ||
        (record.get('outputMode') ===
            dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
        (record.get('outputMode') ===
            dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL)) {
        var advancedForms =
        [
            {
                xtype: 'heatingpropsadvancedform',
                dSUID: record.get('dsuid'),
                outputMode: record.get('outputMode'),
                matrix: matrix
            }
        ];

        forms.push(
        {
            title: dSS._("Advanced settings"),
            forms: advancedForms
        });
    }

    var retObj =
    {
        panel : new dSS.devprops.DevicePropertiesMultitabPanel({
                    DisplayID: record.get('DisplayID'),
                    name: record.get('name'),
                    showDefaults: true,
                    tabForms: forms,
                    loaders: heatingloaders,
                    tabHeight: th
                }),
        extraTitle: dSS._("climate")
    };
    return retObj;
};

dSS.devprops.CreatePanelForConsumptionSettings = function(record) {
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(record);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (matrix.consumption !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Consumption thresholds can only be configured for klemme L and ZWS devices"));
            return;
        }

        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var deviceType = devInfo[0];
        var deviceNumber = devInfo[1];

        var icon = 'titleIcon' + dSS.util.genCSSName(record);

        var thresholdloaders =
            [
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_CURRENT,
                    url: '/json/device/getSensorValue',
                    params:
                    {
                        dsuid: record.get('dsuid'),
                        sensorIndex: 2
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsuid: record.get('dsuid'),
                        eventIndex: 0
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsuid: record.get('dsuid'),
                        eventIndex: 1
                    }
                })
            ];


        var standardForms =
            [
                {
                    xtype: 'currentconsumptionform',
                    dSUID: record.get('dsuid'),
                    device: record
                },
                {
                    xtype: 'consumptioneventform',
                    dSUID: record.get('dsuid'),
                    eventIndex: 0,
                    deviceName: record.get('name'),
                    matrix: matrix,
                    deviceType: deviceType,
                    deviceNumber: deviceNumber
                },
                {
                    xtype: 'consumptioneventform',
                    dSUID: record.get('dsuid'),
                    eventIndex: 1,
                    deviceName: record.get('name'),
                    matrix: matrix,
                    deviceType: deviceType,
                    deviceNumber: deviceNumber
                }
            ];

            if (matrix.consumptioneventled) {
                standardForms.push(
                {
                    xtype: 'consumptionledform',
                    dSUID: record.get('dsuid')
                });

                thresholdloaders.push(
                    new dSS.devprops.Loader({
                        loaderId: dSS.devprops.Loaders.CONSUMPTION_LED_SETTING,
                        url: '/json/device/getConsumptionVisualization',
                        params:
                        {
                            dsuid: record.get('dsuid')
                        }
                    })
                );
            }

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    DisplayID: record.get('DisplayID'),
                    name: record.get('name'),
                    showDefaults: true,
                    standardForms: standardForms,
                    loaders: thresholdloaders,
                    useTabs: false
                })
     };
     return retObj;
};

dSS.devprops.CreatePanelForRenaming = function(record, featureConfig) {
    var forms = [];
    var loaders = [];
    var deviceType;

    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        loaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.METER_NAME,
                url: '/json/circuit/getName',
                params:
                {
                    dsuid: record.get('dsuid')
                }
            })
        );
    } else if (featureConfig === dSS.config.FeatureConfig.ZONE) {
        if (record.get('isFloor') === true) {
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.FLOOR_NAME,
                    url: '/json/structure/getFloorName',
                    params:
                    {
                        floorID: record.get('floorId')
                    }
                })
            );
        } else {
            loaders.push(
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.ZONE_NAME,
                    url: '/json/zone/getName',
                    params:
                    {
                        id: record.get('zoneId')
                    }
                })
            );
        }
    } else {
        loaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.DEVICE_NAME,
                url: '/json/device/getName',
                params:
                {
                    dsuid: record.get('dsuid')
                }
            })
        );
    }

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    name: record.get('name'),
                    showDefaults: false,
                    standardForms:
                    [
                        {
                            xtype: 'dssgenericrenameform',
                            featureConfig: featureConfig,
                            deviceRecord: record
                        }
                    ],
                    loaders: loaders,
                    useTabs: false
                })
     };
    return retObj;
};

dSS.devprops.CreatePanelForEditingGroups = function(record)
{
     var fixedColor = false;
     var deviceStore = dSS.data.Loader.getDeviceStore();
     if (deviceStore.getRange().length > 0) {
         fixedColor = true;
     }

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    name: record.get('name'),
                    showDefaults: false,
                    standardForms:
                    [
                        {
                            xtype: 'dsseditgroupform',
                            group: record,
                            fixedColor: fixedColor
                        }
                    ],
                    useTabs: false
                })
     };
    return retObj;
};

dSS.devprops.CreatePanelForAddingGroups = function(record)
{
    var retObj =
    {
       panel : new dSS.devprops.DevicePropertiesPanel({
                   showDefaults: false,
                   standardForms:
                   [
                       {
                           xtype: 'dssaddgroupform'
                       }
                   ],
                   useTabs: false
               })
    };
    return retObj;
};

dSS.devprops.CreatePanelForSensorValues = function(record, featureConfig)
{

    var allowAssignment = false;
    if (featureConfig === dSS.config.FeatureConfig.ZONE) {
        allowAssignment = true;
    }

    var retObj =
    {
       panel : new dSS.devprops.DevicePropertiesPanel({
                   name: record.get('name'),
                   DisplayID: record.get('DisplayID'),
                   showDefaults: true,
                   standardForms:
                   [
                       {
                           xtype: 'dsssensorpanel',
                           height: 250,
                           standalone: true,
                           zoneAssignment: allowAssignment,
                           dsuid: record.get('dsuid')
                       }
                   ],
                   useTabs: false
               })
    };
    return retObj;
};



dSS.devprops.CreatePropertiesPanel = function(dialogType, record,
                                              featureConfig, scope) {
    switch (dialogType) {
        case dSS.config.dialogTypes.DEVICE_PROPERTIES:
            return dSS.devprops.CreatePanelForDeviceProperties(record,
                                                               featureConfig,
                                                               scope);
        case dSS.config.dialogTypes.SHADE_PROPERTIES:
            return dSS.devprops.CreatePanelForShadeProperties(record);
        case dSS.config.dialogTypes.CONSUMPTION_SETTINGS:
            return dSS.devprops.CreatePanelForConsumptionSettings(record);
        case dSS.config.dialogTypes.RENAME_DIALOG:
            return dSS.devprops.CreatePanelForRenaming(record, featureConfig);
        case dSS.config.dialogTypes.EDIT_GROUP_DIALOG:
            return dSS.devprops.CreatePanelForEditingGroups(record);
        case dSS.config.dialogTypes.CREATE_GROUP_DIALOG:
            return dSS.devprops.CreatePanelForAddingGroups(record);
        case dSS.config.dialogTypes.HEATING_PROPERTIES:
            return dSS.devprops.CreatePanelForHeatingProperties(record);
        case dSS.config.dialogTypes.SENSOR_VALUES:
            return dSS.devprops.CreatePanelForSensorValues(record,
                                                           featureConfig);
        default:
            break;
    }
    // if this happened - there was an illegal use of the framework i.e.
    // programming error, user can't configure anything to ever get here
    return undefined;
};
