//= require <dSS/data/Loader>
//= require <dSS/util/Util>
//= require <dSS/util/DefaultDialog>
//= require <dSS/devices/DevicePropertyLoader>
//= require <dSS/devices/DevicePropertyHelper>
//= require <dSS/devices/DeviceProperties>
//= require <dSS/devices/DeviceSettings>
//= require <dSS/devices/SinglePresence>
//= require <dSS/devices/ConsumptionEventForm>
//= require <dSS/devices/AKMSensorConfig>
//= require <dSS/devices/AKMDelayConfig>
//= require <dSS/devices/AKMInputConfig>

Ext.namespace('dSS', 'dSS.grid', 'dSA', "dSS.gthelper");

// custom parameter "featureConfig"
dSS.grid.DevicePanel = Ext.extend(Ext.grid.GridPanel, {
    constructor: function(config) {
        this.featureConfig = config.featureConfig;
        this.areaSceneSelected = false;
        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            this.removeFromGroupAllowed = false;
        }

        this.allowDeviceValueChange = true;
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            this.allowDeviceValueChange = false;
        }

        var fc = this.featureConfig;

        var binaryInputStatusRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var val = "";
            var valtt = "";
            var productID = record.get('productID');
            var pid = dSS.util.parseProductID(productID);
            var devType = dSS.config.deviceTypes[pid[0]];
            var devNumber = pid[1];


            var returnvalue = function(valtt, val) {
                if (valtt.length > 0) {
                    return String.format('<div ext:qtip="{0}">{1}</div>',
                                        valtt, val);
                } else {
                    return val;
                }
            };

            if (record.get('isVdcDevice') === true) {
                var vdcstates = record.get('vdcOpStates');
                Ext.each(vdcstates, function(obj) {
                        if (val.length > 0) {
                            val = val + ", ";
                        }
                        if (valtt.length > 0) {
                            valtt = valtt + ", ";
                        }

                        // render title only if more states are available
                        // display value data is alerady localized
                        if (vdcstates.length > 1) {
                            val = val + obj.title + ': ' +  obj.displayValue;
                        } else {
                            val = val + obj.displayValue;
                        }
                        valtt = valtt + obj.title + ': ' +  obj.displayValue;
                    }
                );
            }

            var lt = record.get('buttonInputMode');
            if (((devType === 'UMR') && (record.get('dSUIDIndex') < 2) &&
                 ((lt < dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) ||
                  (lt > dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE))) ||
                ((devType === 'TKM') && (devNumber >= 300) &&
                 ((lt < dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) ||
                  (lt > dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE)))) {
                 // UMR is not in sensor mode
                 return returnvalue(valtt, val);
            }

            var inputs = record.get('binaryInputs');
            var i;
            if (inputs === undefined) {
                return returnvalue(valtt, val);
            }

            for (i = 0; i < inputs.length; i++) {
                var input = inputs[i];
                try {
                    var stateLabel = dSS.config.AKMInputTypesValues[input.inputType][input.state];
                    if (stateLabel) {
                        if (val.length > 0) {
                            val = val + ", ";
                        }
                        if (valtt.length > 0) {
                            valtt = valtt + ", ";
                        }
                        val = val + dSS._(stateLabel);
                        valtt = valtt + dSS._(stateLabel);
                    }
                } catch (error) {}
            }

            var sensors = record.get('sensors');
            for (i = 0; i < sensors.length; i++) {
                var sensor = sensors[i];
                if ((sensor.valid === true) &&
                    dSS.util.isVisibleSensorType(sensor.type, true)) {
                    if (val.length > 0) {
                        val = val + ", ";
                    }
                    if (valtt.length > 0) {
                        valtt = valtt + ", ";
                    }

                    val = val + String(Math.round(sensor.value * 10) / 10);
                    valtt = valtt + String(Math.round(sensor.value * 10) / 10);

                    var unit = dSS.config.SensorTypeUnits[sensor.type];
                    if (unit != undefined) {
                        val = val + dSS._(unit);
                        valtt = valtt + dSS._(unit);
                    }
                }
            }
            return returnvalue(valtt, val);
        };

        var commentRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var comment = record.get('comment');

            if ((comment !== undefined) && (comment.length > 0)) {
                return String.format('<div ext:qtip="{0}">{1}</div>',
                        comment, comment);
            } else {
                return "";
            }
        };

        var outputModeRenderer = function(value, metaData, record, rowIndex,
                                          colIndex, store) {
            var functionId = record.get('functionID');
            var mode = record.get('outputMode');
            var val = "";

            var productID = record.get('productID');
            var pid = dSS.util.parseProductID(productID);
            var devType = dSS.config.deviceTypes[pid[0]];
            var outputsCombined = false;


            if (dSS.util.deviceHasOutput(functionId)) {
                if (mode !== undefined) {
                    switch (mode) {
                        case dSS.config.deviceParams.OUTPUT_MODE_DISABLED:
                            val = dSS._("disabled");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_DIMMER:
                            val = dSS._("dimmed");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_LEADING_EDGE_DIMMER:
                            val = dSS._("dimmed (leading edge)");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_RMS_DIMMER:
                            val = dSS._("dimmed - root mean square");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_PULSE_PACKAGE_CTL_DIMMER:
                            val = dSS._("dimmed - pulse package control");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_AUTO:
                            val = dSS._("dimmed (automatic)");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V:
                            val = dSS._("dimmed 0-10V");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V:
                            val = dSS._("dimmed 1-10V");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_POWERSAVE:
                            val = dSS._("Powersave");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_WIPE:
                            val = dSS._("Wipe");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_ALTSWITCH:
                            val = dSS._("Switch");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED:
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH:
                            val = dSS._("switched");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL:
                            if (devType === 'UMR') {
                                val = dSS._("single switched");
                            } else {
                                val = dSS._("switched");
                            }
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH:
                            val = dSS._("combined switched");
                            outputsCombined = true;
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH:
                            val = dSS._("combined two stage switched");
                            outputsCombined = true;
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH:
                            val = dSS._("combined three stage switched");
                            outputsCombined = true;
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM:
                        case dSS.config.deviceParams.OUTPUT_MODE_PWM:
                            val = dSS._("PWM");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_DEFAULT_ACTIVE:
                            val = dSS._("active");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS:
                            val = dSS._("switched with 2 outputs in 2 steps");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS:
                            val = dSS._("switched with 2 outputs in 3 steps");
                            break;
                        case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_PARALLEL:
                            val = dSS._("switched with 2 outputs parallel");
                            break;
                        default:
                            val = "";
                            break;
                    }
                }
            }

            if ((devType === 'UMV') || (devType === 'UMR')) {
                var outputLine = parseInt(record.get('dSUIDIndex'), 10) + 1;
                if (devType === 'UMR') {
                    outputLine -= 2;
                }
                if (!outputsCombined && (val.length > 0)) {
                    val = dSS.gt.strargs(dSS._("Output %1"),
                            [String(outputLine)]) + ", " + val;
                }
            }

            if (fc === dSS.config.FeatureConfig.SCENE) {
                var a = record.get('areas');
                if (a.length > 0) {
                    if (val.length > 0) {
                        val = val + ", ";
                    }
                    val = val + dSS._("Area") + " ";

                    var i;
                    for (i = 0; i < a.length; i++) {
                        val = val + String(a[i]);
                        if (i < a.length - 1) {
                            val = val + ", ";
                        }
                    }
                }
            }

            return val;
        };

        var pbRenderer = function(value, metaData, record, rowIndex, colIndex,
                                  store) {
            var buttonID = record.get('buttonID');
            var val = "";
            var productID = record.get('productID');
            var pid = dSS.util.parseProductID(productID);
            var devType = dSS.config.deviceTypes[pid[0]];
            var devNumber = pid[1];
            var isVdcDevice = record.get('isVdcDevice') === true;

            if ((record.get('buttonUsage') == dSS.config.ButtonUsage.AUTO_UNUSED) ||
                (record.get('buttonUsage') == dSS.config.ButtonUsage.MANUAL_UNUSED)) {
                metaData.css = "unusedInput";
            }

            if (buttonID !== undefined) {
                if ((record.get('productRevision') < dSS.config.PushbuttonMinFW) &&
                    !isVdcDevice) {
                    return val;
                }

                var functionID = record.get('functionID');
                var fid = dSS.util.parseFunctionID(functionID);
                var color = dSS.config.deviceColors[fid[0]];
                if ((color === 'GN') || (color === 'RT')) {
                    if (color === 'GN') {
                        if ((devType === 'TKM') && (devNumber === 200))
                        {
                            return dSS._("leaving / coming home");
                        }

                        if (((devType === 'TKM') && (devNumber === 210)) ||
                            ((devType === 'KM') && (devNumber === 200)) ||
                            ((devType === 'SDS') && (devNumber === 200))) {
                            return dSS._("door bell");
                        }
                    } else if (color == 'RT') {
                        if (((devType === 'TKM') || (devType === 'SDM') ||
                            (devType === 'SDS')) && (devNumber === 200)) {
                            return dSS._("panic");
                        }
                    }
                }

                if (!dSS.util.deviceHasInput(functionID)) {
                    return "";
                }

                var activeGroup = record.get('buttonActiveGroup');

                // #15310
                if (activeGroup === dSS.config.groups.TEMPCONTROL) {
                    return dSS._(dSS.config.groupNames[dSS.config.groups.TEMPCONTROL]);
                } else if (activeGroup === dSS.config.groups.VENTILATION) {
                    return dSS._(dSS.config.groupNames[dSS.config.groups.VENTILATION]);
                } else if (activeGroup === dSS.config.groups.RECIRCULATION) {
                    return dSS._(dSS.config.groupNames[dSS.config.groups.RECIRCULATION]);
                } else if (activeGroup === dSS.config.groups.APT_VENTILATION) {
                    return dSS._(dSS.config.groupNames[dSS.config.groups.APT_VENTILATION]);
                }

                var c;
                if (color === 'SW') {
                    c = dSS.util.getJokerGroup(record.get('groups'));
                    // assume it's not yet configured
                    if (c === null) {
                        c = dSS.config.groups.JOKER;
                    }
                }

                if ((color === 'BL') && (devType === 'KM') &&
                    (devNumber === 200)) {
                    var g = record.get('groups');
                    if (g.indexOf(dSS.config.groups.TEMPCONTROL) !== -1) {
                        return "";
                    }
                }

                if (dSS.util.isUserGroup(activeGroup)) {
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var grec = groupStore.getById(activeGroup);
                    if (grec) {
                        return grec.get('name') + ' (' + dSS._("Group") +
                                '), ' +
                                dSS.util.PushButtonToString(buttonID, (c === dSS.config.groups.JOKER));
                    }
                } else if (dSS.util.isControlGroup(activeGroup)) {
                    return dSS.util.PushButtonToString(buttonID, false);
                }

                var inputs = record.get('binaryInputs');
                var lt = record.get('buttonInputMode');
                if ((isVdcDevice && inputs && inputs.length) ||
                    (devType === 'AKM') ||
                    ((devType === 'UMR') && (record.get('dSUIDIndex') < 2) &&
                     (lt >= dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) &&
                     (lt <= dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE)) ||
                    ((devType === 'TKM') && (devNumber >= 300) &&
                     (lt >= dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD) &&
                     (lt <= dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE))) {
                    if (inputs.length < 1) {
                        return "";
                    }
                    var input = inputs[0];
                    if (input.inputType !== undefined) {
                        val = dSS.config.AKMInputTypeStrings[input.inputType];
                        if (val !== undefined) {
                            if ((devType === 'UMR') ||
                                (devType === 'TKM') && (devNumber >= 300)) {
                                return dSS._("Sensor") + ", " + dSS._(val);
                            }
                            return dSS._(val);
                        } else {
                            return "";
                        }
                    }
                }

                if (lt == dSS.config.deviceParams.BUTTONINPUT_DISABLED) {
                    return dSS._("disabled");
                }
                val = dSS.util.PushButtonToString(buttonID, (c === dSS.config.groups.JOKER));
            }

            return val;
        };

        var buttonInputRenderer = function(value, metaData, record, rowIndex, colIndex,
                                  store) {
            var buttonMode = record.get('buttonInputMode');
            var buttonIndex = record.get('buttonInputIndex');
            var buttonCount = record.get('buttonInputCount');
            var val = "";
            if (buttonMode !== undefined) {
                var fid = dSS.util.parseFunctionID(record.get('functionID'));
                var jokerGroup;
                if (fid[0] === dSS.config.deviceClass.SW) {
                    jokerGroup = dSS.util.getJokerGroup(record.get('groups'));
                }
                var productID = record.get('productID');
                var pid = dSS.util.parseProductID(productID);
                var devType = dSS.config.deviceTypes[pid[0]];
                if (devType === 'AKM') {
                    val = dSS.util.AKMModeToString(record.get('AKMInputProperty'),
                                                   buttonIndex,
                                                   buttonCount);
                } else if ((devType === 'UMR') && (record.get('dSUIDIndex') > 1)) {
                    val = "";
                } else {
                    val = dSS.util.PushButtonModeToString(buttonMode,
                                                      buttonIndex,
                                                      buttonCount, jokerGroup,
                                                      (devType === 'SDS'));
                }
            }

            return val;
        };

        var dSUIDmeterRenderer= function(value, metaData, record, rowIndex, colIndex, store) {
            var shortId = dSS.util.shortDSUID(record.get('meterDSUID'));
            return String.format('<div ext:qtip="dSUID: {0}">{1}</div>',
                                  record.get('meterDSUID'), shortId);
        };

        var defaultDateRenderer = Ext.util.Format.dateRenderer('Y-m-d\\&\\n\\b\\s\\p\\;H:i:s');

        var inactiveSinceRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var d = record.get('inactiveSince');
            if ((d == undefined) || (d.getFullYear() == 1970)) {
                return "";
            } else {
                return defaultDateRenderer(d);
            }
        };

        var userGroupRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            var deviceGroups = record.get('groups');
            var groupNames = "";
            var addBreak = false;
            var groupStore = dSS.data.Loader.getGroupStore();
            Ext.each(deviceGroups, function(group) {
                if (!dSS.util.isUserGroup(group)) {
                    return true;
                }
                var gr = groupStore.getById(group);
                if (gr) {
                    var gn = gr.get('name');
                    if (gn.length > 0) {
                        if (addBreak) {
                            groupNames = groupNames + '<br/>';
                        }
                        groupNames = groupNames + gn;
                        addBreak = true;
                    }
                }
            });
            return groupNames;
        };

        var hideColumn = false;
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            hideColumn = true;
        }

        var deviceCols = [
        {
            header: "",
            width: 20,
            resizable: false,
            sortable: false,
            hideable: false,
            menuDisabled: true,
            fixed: true,
            dataIndex: 'functionID',
            renderer: dSS.util.gridDeviceTypeRenderer,
            id: 'functionID'
        },
        {
            header: dSS._("Name"),
            tooltip: dSS._("Name"),
            width: 80,
            sortable: true,
            dataIndex: 'name',
            renderer: dSS.util.gridDeviceNameRenderer,
            id: 'name'
        },
        {
            header: dSS._("ID"),
            tooltip: dSS._("ID"),
            width: 70,
            sortable: true,
            dataIndex: 'dsuid',
            id: 'dsuid',
            renderer: dSS.util.gridDeviceDSUIDRenderer
        }];

        deviceCols.push(
            {
                header: dSS._("Output mode"),
                tooltip: dSS._("Output mode"),
                sortable: true,
                dataIndex: 'outputMode',
                id: 'outputMode',
                renderer: outputModeRenderer
            });

        deviceCols.push(
            {
                header: dSS._("Input mode"),
                tooltip: dSS._("Input mode"),
                sortable: true,
                dataIndex: 'buttonID',
                id: 'buttonID',
                tdCls: 'x-pbcell',
                renderer: pbRenderer
            });

        if (this.featureConfig !== dSS.config.FeatureConfig.GROUP) {
            deviceCols.push(
                {
                    header: dSS._("Input"),
                    tooltip: dSS._("Input"),
                    hidden: hideColumn,
                    sortable: true,
                    dataIndex: 'buttonMode',
                    id: 'buttonMode',
                    renderer: buttonInputRenderer
                });
        } else {
            deviceCols.push(
                {
                    header: dSS._("User groups"),
                    tooltip: dSS._("User groups"),
                    hidden: hideColumn,
                    sortable: true,
                    dataIndex: 'groups',
                    id: 'userGroups',
                    renderer: userGroupRenderer
                });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols.push(
                {
                    header: dSS._("Status"),
                    tooltip: dSS._("Status"),
                    hidden: hideColumn,
                    sortable: true,
                    dataIndex: 'binaryInputs',
                    renderer: binaryInputStatusRenderer
                });
        }

        if (this.featureConfig !== dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols.push(
            {
                header: dSS._("dSM Name"),
                tooltip: dSS._("dSM Name"),
                width: 80,
                sortable: true,
                dataIndex: 'meterName',
                id: 'meterName'
            });
        }

        deviceCols.push(
        {
            header: dSS._("Room name"),
            tooltip: dSS._("Room name"),
            hidden: hideColumn,
            width: 80,
            sortable: true,
            dataIndex: 'zoneName',
            id: 'zoneName',
            renderer: dSS.util.gridZoneNameRenderer
        });

        if ((this.featureConfig === dSS.config.FeatureConfig.SENSOR) ||
            (this.featureConfig === dSS.config.FeatureConfig.ZONE)) {
            var sensorAssignmentRenderer = function(value, metaData, record,
                                                    rowIndex, colIndex, store) {
                var sensors = dSS.util.UniqueArray(record.get('assignedSensorTypes'), true);
                if (sensors.length > 0) {
                    var str = "";
                    var i;
                    for (i = 0; i < sensors.length; i++) {
                        var unit = String(dSS.config.SensorTypeUnits[sensors[i]]);
                        if (unit.length > 0) {
                            str = str + dSS._(unit);
                        }

                        if (i < sensors.length - 1) {
                            str = str + ", ";
                        }
                    }
                    return str;
                } else {
                    return "";
                }
            };

            deviceCols.push(
            {
                header: dSS._("Room sensor"),
                tooltip: dSS._("Room sensor"),
                width: 45,
                sortable: true,
                dataIndex: 'assignedSensorTypes',
                renderer: sensorAssignmentRenderer
            });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            deviceCols = deviceCols.concat(
            [
                {
                    header: dSS._("Room ID"),
                    tooltip: dSS._("Room ID"),
                    width: 35,
                    sortable: true,
                    dataIndex: 'zoneId',
                    id: 'zoneId'
                },
                {
                    header: dSS._("HW info"),
                    tooltip: dSS._("Hardware info"),
                    width: 50,
                    sortable: true,
                    id: 'HW',
                    dataIndex: 'HW'
                },
                {
                    header: dSS._("Firmware version"),
                    tooltip: dSS._("Firmware version"),
                    width: 50,
                    sortable: true,
                    id: 'fw',
                    dataIndex: 'firmwareVersion'
                },
                {
                    header: dSS._("Registered on"),
                    tooltip: dSS._("Registered on"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'firstSeen',
                    xtype: 'datecolumn',
                    format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:i:s',
                    id: 'firstSeen'
                },
                {
                    header: dSS._("Inactive since"),
                    tooltip: dSS._("Inactive since"),
                    width: 60,
                    sortable: true,
                    dataIndex: 'inactiveSince',
                    renderer: inactiveSinceRenderer,
                    id: 'inactiveSince'
                }
            ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) ||
            (this.featureConfig === dSS.config.FeatureConfig.GROUP)) {
            deviceCols.push(
                {
                    header: dSS._("Comment"),
                    tooltip: dSS._("Comment"),
                    hidden: hideColumn,
                    width: 20,
                    resizable: false,
                    sortable: false,
                    dataIndex: 'comment',
                    id: 'comment',
                    renderer: commentRenderer
                });
        }

        var selModel;

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.SENSOR) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: true });
        } else {
            selModel = new Ext.grid.RowSelectionModel({ singleSelect: false });
        }

        var enabledd = false;
        var ddgroup = "";
        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            enabledd = true;
            ddgroup = "zoneDeviceDD";
        } else if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            enabledd = true;
            ddgroup = "groupDeviceDD";
        }

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: deviceCols,
            ddGroup: ddgroup,
            bubbleEvents: [ 'removedfromgroup' ],
            enableDragDrop: enabledd,
            stripeRows: true,
            title: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? undefined :  dSS._("Device", "Devices", 2),
            selModel: selModel,
            viewConfig: {
                forceFit: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }

                    var locked = record.get('locked');
                    if (locked === true) {
                        return 'lockedDevice';
                    }

                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        this.selectionBlocked = true;

        dSS.grid.DevicePanel.superclass.constructor.apply(this, arguments);

        this.addEvents('removedfromgroup');

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            dSS.MessageBus.on('setdevicename',
                    dSS.util.setName.createDelegate(this),
            this);
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            dSS.MessageBus.on('sceneselected', this.handleSceneSelect.createDelegate(this),
            this);

            dSS.MessageBus.on('updateareadata', function(data) {
                    var store = this.getStore();
                    var i;
                    for (i = 0; i < data.length; i++) {
                        var device = data[i];
                        if (device.areas.length > 0) {
                            var d = store.getById(device.dsuid);
                            if (d) {
                                d.set('areas', device.areas);
                                d.commit();
                            }
                        }
                    }
                    store.commitChanges();
                }, this);
        }

        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            this.on('afterrender', function() {
                    var gv = this.getView();
                    this.origb4Drag = gv.dragZone.b4Drag.createDelegate(gv.dragZone);

                    gv.dragZone.b4Drag = function(e) {
                        var store = this.grid.getStore();
                        var sel = this.grid.getSelectionModel();
                        var selected = sel.getSelections();
                        var i;
                        for (i = 0; i < selected.length; i++) {
                            var record = selected[i];
                            if (record.get('isPresent') === false) {
                                var index = store.findExact('dsuid',
                                        record.get('dsuid'));
                                if (index > -1) {
                                    sel.deselectRow(index, false);
                                }
                            }
                        }
                        selected = sel.getSelections();
                        if (selected.length > 0) {
                            var proxy = this.getProxy();
                            var ghost = proxy.getGhost();
                            ghost.update(dSS.gt.strargs(
                                            dSS.gt.ngettext(
                                                "%1 device selected",
                                                "%1 devices selected",
                                                selected.length),
                                            [ selected.length ]));
                            this.grid.origb4Drag(e);
                        }
                    };
                    gv.dragZone.onBeforeDrag = function(data, evt) {
                        var i;
                        var retval = false;
                        for (i = 0; i < data.selections.length; i++) {
                            var record = data.selections[i];
                            if (record.get('isPresent') === true) {
                                retval = true;
                            }
                        }
                        return retval;
                    };
                },
                this);
        }


        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            this.getSelectionModel().on('beforerowselect',
                    function(sel, rowIndex, keepExisting, record) {
                        // no extra checks needed for single select
                        if (keepExisting === false) {
                            return true;
                        }

                        // do not allow mixed color multiple selections
                        var previous = sel.getSelected();
                        if (previous === undefined) {
                            return true;
                        }


                        // AKMs have no color
                        var pid = dSS.util.parseProductID(
                                                record.get('productID'));
                        if (pid[0] === dSS.config.deviceTypeNum.AKM) {
                            return true;
                        }

                        pid = dSS.util.parseProductID(
                                                previous.get('productID'));
                        // if previous selection is an AKM also we have to
                        // check the whole range, first non AKM will define
                        // the colora
                        var nonAKM = false;
                        if (pid[0] === dSS.config.deviceTypeNum.AKM) {
                            var range = sel.getSelections();
                            Ext.each(range, function(rec) {
                                pid = dSS.util.parseProductID(
                                                        rec.get('productID'));
                                if (pid[0] !== dSS.config.deviceTypeNum.AKM) {
                                    nonAKM = true;
                                    previous = rec;
                                    // abort loop
                                    return false;
                                }
                            });
                        } else {
                            nonAKM = true;
                        }

                        if (nonAKM !== true) {
                            return true;
                        }

                        // for all others the color has to match
                        previouscolor = dSS.util.getDeviceColor(previous);
                        var currentcolor = dSS.util.getDeviceColor(record);
                        if (previouscolor !== currentcolor) {
                            return false;
                        }

                        return true;
                },
                this);

            dSS.MessageBus.on('groupselection', function(group, allowrm) {
                    var sel = this.getSelectionModel();
                    var tunebar = allowrm;

                    if (!sel.hasSelection()) {
                        tunebar = false;
                    }

                    this.getTopToolbar().removeFromGroupButton.setDisabled(!tunebar);
                    this.removeFromGroupAllowed = allowrm;
                    this.selectedGroup = group;
                },
                this);
        }

        this.on('rowclick', this.handleDeviceRowSelect.createDelegate(this), this);

        dSS.MessageBus.on('structureloaded',
                this.setInitialSelection.createDelegate(this), this);

        this.on('viewready', function() {
            this.selectionBlocked = false;
            var fn = this.setInitialSelection.createDelegate(this);
            fn();
        }, this);

        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            dSS.MessageBus.on('filter_devices_zone',
                this.filterDevices.createDelegate(this), this);
        } else if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            dSS.MessageBus.on('filter_devices_circuit',
                this.filterDevices.createDelegate(this), this);
        } else if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
            dSS.MessageBus.on('filter_devices_sensor',
                this.filterDevices.createDelegate(this), this);
        } else if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            dSS.MessageBus.on('filter_devices_group',
                this.filterDevices.createDelegate(this), this);
        }

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
                this.handleDeviceRowSelect(grid, rowIndex, event);
            }

            var menuItem;
            var showRemoveMenuItem = false;
            var showUnlockDeviceMenuItem = false;
            var presentInSelection = false;
            var selectedDeviceIsPresent = false;
            var deviceHasOutput = false;
            var showRemoveFromGroupMenuItem = false;
            var canBlink = true;
            
            var matrix;
            try {
            	matrix = dSS.devprops.GetVisibleProperties(sel.getSelected());
            }
            catch(error) {
            	matrix = error;
            }            
            
            if (sel.getSelected().get("isPresent") === true) {
                selectedDeviceIsPresent = true;
            }

            var fidUnparsed = sel.getSelected().get("functionID");
            if (dSS.util.deviceHasOutput(fidUnparsed) === true) {
                deviceHasOutput = true;
            }

            var fid = dSS.util.parseFunctionID(fidUnparsed);
            var vdcDevice = sel.getSelected().get('isVdcDevice');
            
            var devInfo = dSS.util.parseProductID(sel.getSelected().get('productID'));
            var devType = devInfo[0];
            var devNumber = devInfo[1];

            if (!matrix.identification) {
            	canBlink = false;
            }

            var white = ((fid[0] == dSS.config.deviceClass.WE) && vdcDevice);

            var records = sel.getSelections();

            var i;
            for (i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                } else {
                    presentInSelection = true;
                    showRemoveFromGroupMenuItem = true;
                }

                if ((record.get('locked') === true) &&
                    (record.get('isPresent') === true))
                {
                    showUnlockDeviceMenuItem = true;
                }
            }

            menuItem = new Ext.menu.Item({
                text: dSS._("Identification Blink"),
                iconCls: 'blink',
                handler: this.blinkDevice.createDelegate(this, rowIndex,
                                                         true),
                disabled: !presentInSelection || !canBlink
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Edit device values"),
                iconCls: 'editValueToolbarIcon',
                handler: this.showControlDeviceWindow.createDelegate(
                                                this, rowIndex, true),
                disabled: !selectedDeviceIsPresent,
                hidden: !this.allowDeviceValueChange
            });
            this.contextMenu.add(menuItem);

            this.contextMenu.add('-');

            if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Assign room sensor"),
                    iconCls: 'assignSensorToZone',
                    disabled: this.checkDisableSVButton(sel.getSelected()),
                    handler: this.showSensorValues.createDelegate(
                                                this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Add to area"),
                    iconCls: 'addToAreaIcon',
                    handler: this.addToArea.createDelegate(this, rowIndex, true),
                    disabled: !this.areaSceneSelected || !deviceHasOutput
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove from area"),
                    iconCls: 'removeFromAreaIcon',
                    handler: this.removeFromArea.createDelegate(this, rowIndex, true),
                    disabled: !this.areaSceneSelected || !deviceHasOutput
                });
                this.contextMenu.add(menuItem);

                this.contextMenu.add('-');
            }

            if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Test transmission quality"),
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(selectedDeviceIsPresent && !vdcDevice)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Update device status"),
                    iconCls: 'checkDevicePresenceIcon',
                    handler: this.getDevicePresence.createDelegate(
                                                    this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);

                this.contextMenu.add('-');

                menuItem = new Ext.menu.Item({
                    text: dSS._("Show sensor values"),
                    iconCls: 'showSensorValues',
                    disabled: (this.checkDisableSVButton(sel.getSelected()) || !selectedDeviceIsPresent) ,
                    handler: this.showSensorValues.createDelegate(
                                                this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            var dr = sel.getSelected();

            if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                var disabled = false

                try {
                    dSS.devprops.GetVisibleProperties(dr);
                } catch (error) {
                    disabled = true;
                }

                // special case for single group devices
                if (white) {
                    disabled = false;
                }

                if (!selectedDeviceIsPresent) {
                    disabled = true;
                }

                menuItem = new Ext.menu.Item({
                text: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? dSS._("Edit activity settings") : dSS._("Edit device properties"),
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: disabled
                });
                this.contextMenu.add(menuItem);


            }

            if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
                (this.featureConfig === dSS.config.FeatureConfig.GROUP)) {
                var disabled = false
                try {
                    dSS.devprops.GetVisibleProperties(dr);
                } catch (error) {
                    disabled = true;
                }

                // special case for single group devices
                if (white) {
                    disabled = false;
                }

                if (!selectedDeviceIsPresent) {
                    disabled = true;
                }

                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit device properties"),
                    iconCls: 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow2.createDelegate(
                                                this, rowIndex, true),
                    disabled: disabled
                });
                this.contextMenu.add(menuItem);
            }

            if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                var matrix;
                var outputMode = sel.getSelected().get("outputMode");

                try {
                    matrix = dSS.devprops.GetVisibleProperties(dr);
                } catch (error) {
                    // for now we can safely assume all as invisible
                    matrix = error;
                }

                var showHeatingProps =
                    (selectedDeviceIsPresent && matrix.heatingprops) &&
                    (dSS.util.isValidHeatingOutputMode(outputMode) ||
                    ((devType === dSS.config.deviceTypeNum.ZWS) &&
                     (devNumber === 205) &&
                     dSS.util.getHeatingGroup(sel.getSelected().get('groups')) == dSS.config.groups.TEMPCONTROL));

                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit shade device properties"),
                    iconCls: 'shadePropertiesIcon',
                    handler: this.showShadePropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(selectedDeviceIsPresent && matrix.shadeprops)
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit climate device properties"),
                    iconCls: 'climatePropertiesIcon',
                    handler: this.showHeatingPropertiesWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(showHeatingProps && (record.get('VdcHardwareModelGuid') === ''))
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Configure consumption event"),
                    iconCls: 'consumptionEventIcon',
                    handler: this.showConsumptionThresholdWindow.createDelegate(
                                                    this, rowIndex, true),
                    disabled: !(selectedDeviceIsPresent && matrix.consumption)
                });
                this.contextMenu.add(menuItem);
            }

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename device"),
                iconCls: 'editIcon',
                handler: this.renameDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.GROUP) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
                menuItem = new Ext.menu.Item({
                    text: dSS._("Edit comment"),
                    iconCls: 'editCommentIcon',
                    handler: this.editComment.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
                (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {

                this.contextMenu.add('-');

                menuItem = new Ext.menu.Item({
                    text: dSS._("Unlock device"),
                    iconCls: 'unlockDeviceIcon',
                    handler: this.unlockDevices.createDelegate(this, rowIndex,
                                                              true),
                    disabled: !showUnlockDeviceMenuItem
                });
                this.contextMenu.add(menuItem);

                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove device"),
                    iconCls: 'removeDeviceIcon',
                    handler: this.removeDevice.createDelegate(this, rowIndex,
                                                              true),
                    disabled: !showRemoveMenuItem
                });
                this.contextMenu.add(menuItem);
            }


            if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
                this.contextMenu.add('-');

                if (this.removeFromGroupAllowed !== true) {
                    showRemoveFromGroupMenuItem = false;
                }
                menuItem = new Ext.menu.Item({
                    text: dSS._("Remove device from user group"),
                    iconCls: 'removeFromAreaIcon',
                    handler: this.removeDeviceFromGroup.createDelegate(
                                                        this, rowIndex, true),
                    disabled: !showRemoveFromGroupMenuItem
                });
                this.contextMenu.add(menuItem);
            }

            var current = sel.getSelected();
            if (current) {
                var url = String(current.get('VdcConfigURL'));
                if (url.length > 0) {
                    this.contextMenu.add('-');
                    menuItem = new Ext.menu.Item({
                        text: dSS._("Open vdc interface"),
                        iconCls: 'openVdcInterface',
                        href: url,
                        hrefTarget: '_blank'
                    });
                    this.contextMenu.add(menuItem);
                }
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    setInitialSelection: function() {
        if (this.selectionBlocked === true) {
            return;
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            return;
        }

        if (!this.isVisible()) {
            return;
        }

        if (!this.ownerCt.isVisible()) {
            return;
        }

        if (this.ownerCt.ownerCt) {
            if (!this.ownerCt.ownerCt.isVisible()) {
                return;
            }
        }

        if (this.getStore().getCount() === 0) {
            return;
        }

        var selModel = this.getSelectionModel();
        var record;
        var index;
        var selected = false;

        if (selModel.hasSelection()) {
            record = selModel.getSelected();
            index = this.getStore().indexOf(record);
        } else if (this.lastSelectedDSUID !== undefined) {
            index = this.getStore().findExact('dsuid', this.lastSelectedDSUID);
            if (index != -1) {
                record = this.getStore().getAt(index);
            }
        }

        if (record === undefined) {
            record = this.getStore().getAt(0);
            index = 0;
        }

        if (record !== undefined) {
            this.lastSelectedDSUID = record.get('dsuid');
            if (!selModel.hasSelection()) {
                this.getView().focusRow(index);
                selModel.selectRecords([record], false);
            }
            if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
                dSS.MessageBus.fireEvent('loadsensordata', record.get('dsuid'));
                dSS.MessageBus.fireEvent('loadbinaryinputdata',
                                         record.get('dsuid'));
            }
        }
        this.handleDeviceRowSelect();
    },
    buildTopToolbar: function() {
        var bar = [];

        bar.push({
            iconCls: 'blink',
            handler: this.blinkDevice,
            tooltip: dSS._("Blink to identify device"),
            ref: 'blinkButton',
            scope: this
        });

        bar.push({
            iconCls: 'editValueToolbarIcon',
            handler: this.showControlDeviceWindow,
            tooltip: dSS._("Edit device values"),
            ref: 'editValueButton',
            hidden: !this.allowDeviceValueChange,
            scope: this
        });

        bar.push('-');

        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            bar.push({
                iconCls: 'assignSensorToZone',
                handler: this.showSensorValues,
                tooltip: dSS._("Assign room sensor"),
                ref: 'assignSensorsButton',
                scope: this
            });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            bar = bar.concat(
            [
                {
                    ref: 'addToAreaButton',
                    iconCls: 'addToAreaIcon',
                    handler: this.addToArea,
                    tooltip: dSS._("Add to area"),
                    scope: this,
                    disabled: true
                },
                {
                    ref: 'removeFromAreaButton',
                    iconCls: 'removeFromAreaIcon',
                    handler: this.removeFromArea,
                    tooltip: dSS._("Remove from area"),
                    scope: this,
                    disabled: true
                },
                '-'
           ]);
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            bar = bar.concat(
            [
                {
                    iconCls: 'transmissionQuality',
                    handler: this.getTransmissionQuality,
                    tooltip: dSS._("Test transmission quality"),
                    ref: 'transmissionQualityButton',
                    scope: this
                },
                {
                    iconCls: 'checkDevicePresenceIcon',
                    handler: this.getDevicePresence,
                    tooltip: dSS._("Update device status"),
                    scope: this
                },
                '-',
                {
                    iconCls: 'showSensorValues',
                    handler: this.showSensorValues,
                    tooltip: dSS._("Show sensor values"),
                    ref: 'showSensorValuesButton',
                    scope: this
                }
           ]);
        }

        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
                {
                    iconCls: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? 'sceneDevicePropertiesIcon' : 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow,
                    tooltip: (this.featureConfig === dSS.config.FeatureConfig.SCENE) ? dSS._("Edit activity settings") : dSS._("Configure device properties"),
                    ref: 'devicePropertiesButton',
                    scope: this
                });
        }

        // #5415
        if ((this.featureConfig === dSS.config.FeatureConfig.SCENE) ||
            (this.featureConfig === dSS.config.FeatureConfig.GROUP))
        {
            bar.push(
                {
                    iconCls: 'devicePropertiesIcon',
                    handler: this.showDevicePropertiesWindow2,
                    tooltip: dSS._("Configure device properties"),
                    ref: 'devicePropertiesButtonScene',
                    scope: this
                });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            bar.push(
                {
                    iconCls: 'shadePropertiesIcon',
                    handler: this.showShadePropertiesWindow,
                    tooltip: dSS._("Edit shade device properties"),
                    ref: 'shadePropertiesButton',
                    scope: this
                },
                {
                    iconCls: 'climatePropertiesIcon',
                    handler: this.showHeatingPropertiesWindow,
                    tooltip: dSS._("Edit climate device properties"),
                    ref: 'heatingPropertiesButton',
                    scope: this
                },
                {
                    iconCls: 'consumptionEventIcon',
                    handler: this.showConsumptionThresholdWindow,
                    tooltip: dSS._("Configure consumption event"),
                    ref: 'consumptionEventButton',
                    scope: this
                });
        }

        bar.push(
        {
            iconCls: 'renameDeviceToolbarIcon',
            handler: this.renameDevice,
            tooltip: dSS._("Rename device"),
            scope: this
        });

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) ||
            (this.featureConfig === dSS.config.FeatureConfig.GROUP)) {
            bar.push(
            {
                iconCls: 'editCommentToolbarIcon',
                handler: this.editComment,
                tooltip: dSS._("Edit comment"),
                scope: this
            });
        }

        if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            bar.push(
            {
                iconCls: 'removeFromAreaIcon',
                handler: this.removeDeviceFromGroup,
                tooltip: dSS._("Remove device from user group"),
                ref: 'removeFromGroupButton',
                scope: this
            });
        }


        bar.push('->');

        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            bar.push(
            {
                iconCls: 'unlockDeviceToolbarIcon',
                handler: this.unlockDevices,
                tooltip: dSS._("Unlock device"),
                ref: 'unlockDeviceButton',
                disabled: true,
                scope: this
            });

            bar.push(
            {
                iconCls: 'removeDeviceToolbarIcon',
                handler: this.removeDevice,
                tooltip: dSS._("Remove device"),
                ref: 'removeDeviceButton',
                scope: this
            });
        }

        return bar;
    },

    showControlDeviceWindow: function(item, event, rowIndex) {
        var records = this.getStore().getRange();
        var sel = this.getSelectionModel();
        var index;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            var dSUID = sel.getSelected().get('dsuid');
            var i;
            for (i = 0; i < records.length; i++) {
                if (records[i].get('dsuid') === dSUID) {
                    index = i;
                    break;
                }
            }
        } else {
            index = rowIndex;
        }

        var win = new dSS.devices.DeviceSettingsWindow({
            devices: records,
            index: index,
            rowSel: sel,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show(this);
    },
    handleSceneSelect: function(scene) {
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            if (dSS.util.isAreaScene(scene)) {
                this.areaSceneSelected = true;
                var sel = this.getSelectionModel();
                if (sel.hasSelection()) {
                    var record = sel.getSelected();
                    if (dSS.util.deviceHasOutput(record.get('functionID'))) {
                        this.getTopToolbar().addToAreaButton.enable();
                        this.getTopToolbar().removeFromAreaButton.enable();
                        return;
                    }
                }

            } else {
                this.areaSceneSelected = false;
            }
            this.getTopToolbar().addToAreaButton.disable();
            this.getTopToolbar().removeFromAreaButton.disable();
        }
    },
    handleDeviceRowSelect: function(grid, rowIndex, event) {
        var sel = this.getSelectionModel();
        var tb = this.getTopToolbar();

        if (!sel.hasSelection()) {
            if (this.featureConfig === dSS.config.FeatureConfig.GROUP)  {
                tb.removeFromGroupButton.setDisabled(true);
            }
            return;
        }
        var rec = sel.getSelected();
        var present = rec.get("isPresent");
        var vdcDevice = rec.get('isVdcDevice');
        var fid = dSS.util.parseFunctionID(rec.get('functionID'));
        var canBlink = true;

        var white = ((fid[0] == dSS.config.deviceClass.WE) && vdcDevice);

        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            if ((dSS.util.deviceHasOutput(sel.getSelected().get("functionID")) === true) &&
                (this.areaSceneSelected === true)) {
                tb.addToAreaButton.enable();
                tb.removeFromAreaButton.enable();
            } else {
                tb.addToAreaButton.disable();
                tb.removeFromAreaButton.disable();
            }
            try {
                dSS.devprops.GetVisibleProperties(rec);
                tb.devicePropertiesButton.setDisabled(false);
                tb.devicePropertiesButtonScene.setDisabled(false);
            } catch (error) {
                tb.devicePropertiesButton.setDisabled(true);
                tb.devicePropertiesButtonScene.setDisabled(true);
            }
            // special case for single group devices
            if (white && present) {
                tb.devicePropertiesButton.setDisabled(false);
                tb.devicePropertiesButtonScene.setDisabled(false);
            }
        } else {
            this.lastSelectedDSUID = sel.getSelected().get('dsuid');
        }
        
        var matrix;
        try {
        	matrix = dSS.devprops.GetVisibleProperties(rec);
        }
        catch(error) {
        	matrix = error;
        }
        
        if (!matrix.identification) {
        	canBlink = false;
        }

        this.getTopToolbar().blinkButton.setDisabled(!present || !canBlink);
        var b = this.getTopToolbar().editValueButton;
        if (b !== undefined) {
            b.setDisabled(!present);
        }
        if ((this.featureConfig === dSS.config.FeatureConfig.ZONE) ||
            (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT)) {
            tb.removeDeviceButton.setDisabled(present);

            var locked = sel.getSelected().get("locked");
            if ((present === true) && (locked === true)) {
                tb.unlockDeviceButton.setDisabled(false);
            } else {
                tb.unlockDeviceButton.setDisabled(true);
            }
        }
        if (this.featureConfig === dSS.config.FeatureConfig.GROUP)  {
            if (present === true) {
                if ((this.selectedGroup === undefined) ||
                    (!dSS.util.isUserGroup(this.selectedGroup.get('id'))) ||
                    ((dSS.util.isUserGroup(this.selectedGroup.get('id')) === true) &&
                     (this.selectedGroup.get('locked') == true))) {
                    tb.removeFromGroupButton.setDisabled(true);
                } else {
                    tb.removeFromGroupButton.setDisabled(false);
                }
            } else {
                tb.removeFromGroupButton.setDisabled(true);
            }
        }

        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            if (present === true) {
                tb.assignSensorsButton.setDisabled(this.checkDisableSVButton(rec));
            } else {
                tb.assignSensorsButton.disable();
            }
        }

        if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            tb.transmissionQualityButton.setDisabled(!present);
            tb.devicePropertiesButton.setDisabled(!present);
            tb.consumptionEventButton.setDisabled(!present);

            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(rec);
            } catch (error) {
                // for now we can safely assume all as invisible
                matrix = error;
                // special case, dont block properties for single group
                // devices: dialog will opena link to another page
                if (white && present) {
                    tb.devicePropertiesButton.setDisabled(false);
                } else {
                    tb.devicePropertiesButton.setDisabled(true);
                }
            }
            var outputMode = sel.getSelected().get("outputMode");

            var devInfo = dSS.util.parseProductID(sel.getSelected().get('productID'));
            var devType = devInfo[0];
            var devNumber = devInfo[1];

            var showHeatingProps =
                (present && matrix.heatingprops) &&
                (dSS.util.isValidHeatingOutputMode(outputMode) ||
                ((devType === dSS.config.deviceTypeNum.ZWS) &&
                 (devNumber === 205) &&
                 dSS.util.getHeatingGroup(sel.getSelected().get('groups')) == dSS.config.groups.TEMPCONTROL));

            if (present === true) {

                tb.shadePropertiesButton.setDisabled(!matrix.shadeprops);
                tb.heatingPropertiesButton.setDisabled(!(showHeatingProps && (sel.getSelected().get('VdcHardwareModelGuid') === '')));
                tb.consumptionEventButton.setDisabled(!matrix.consumption);
                tb.showSensorValuesButton.setDisabled(this.checkDisableSVButton(rec));
                tb.transmissionQualityButton.setDisabled(vdcDevice);
            } else {
                tb.shadePropertiesButton.disable();
                tb.heatingPropertiesButton.disable();
                tb.consumptionEventButton.disable();
                tb.showSensorValuesButton.disable();
                tb.transmissionQualityButton.disable();
            }
        }

        if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
            dSS.MessageBus.fireEvent('loadsensordata', 
                                     sel.getSelected().get('dsuid'));
            dSS.MessageBus.fireEvent('loadbinaryinputdata', 
                                     sel.getSelected().get('dsuid'));
        }
    },
    renameDevice: function(item, event, rowIndex) {
        var record;

        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device " +
                          "from the list!"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            title: dSS._("Rename device"),
            jumpMode: true,
            featureConfig: dSS.config.FeatureConfig.DEVICE,
            callerScope: this,
            width: 350,
            dialogType: dSS.config.dialogTypes.RENAME_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    addToArea: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var scene;
        try {
            scene = this.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        if (!dSS.util.isAreaScene(scene)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device can only be added to area scenes!"));
            return;
        }

        dSS.util.genericRequest({
                url: '/json/device/addToArea',
                params: {
                    dsuid: record.get('dsuid'),
                    areaScene: scene
                },
                failureMsg: dSS._("Could not add device to area"),
                scope: this,
                maskEl: this.findParentByType(Ext.Viewport).el,
                maskMsg: dSS.gt.strargs(
                             dSS._("Adding device %1 to %2"),
                             [
                                 '&quot;' +
                                 dSS.util.getNameOrDisplayID(record.get('name'),
                                                      record.get('DisplayID')) +
                                 '&quot;',
                                 dSS._(dSS.config.sceneNamesMsg[scene])
                             ]
                         )
            },
            function(success) {
                if (success === true) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Device %1 was added to %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDisplayID(record.get('name'),
                                                      record.get('DisplayID')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
                    );

                    // area on scenes
                    if ((scene >= 6) && (scene <= 9)) {
                        var areas = record.get('areas').splice(0);
                        areas.push(scene - 5);
                        record.set('areas', dSS.util.UniqueArray(areas, true));
                        record.commit();
                    }
                }

                dSS.MessageBus.fireEvent('reloaddevicescenevalue');
            });
    },
    removeFromArea: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var scene;
        try {
            scene = this.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        if (!dSS.util.isAreaScene(scene)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device can only be removed from area scenes!"));
            return;
        }

        dSS.util.genericRequest({
                url: '/json/device/removeFromArea',
                params: {
                    dsuid: record.get('dsuid'),
                    areaScene: scene
                },
                failureMsg: dSS._("Could not remove device from area"),
                scope: this,
                maskEl: this.findParentByType(Ext.Viewport).el,
                maskMsg: dSS.gt.strargs(
                            dSS._("Removing device %1 from %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDisplayID(record.get('name'),
                                                     record.get('DisplayID')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
            },
            function(success) {
                if (success === true) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Device %1 was removed from %2"),
                            [
                                '&quot;' +
                                dSS.util.getNameOrDisplayID(record.get('name'),
                                                     record.get('DisplayID')) +
                                '&quot;',
                                dSS._(dSS.config.sceneNamesMsg[scene])
                            ]
                        )
                    );

                    // area on scenes
                    if ((scene >= 6) && (scene <= 9)) {
                        var areas = record.get('areas').splice(0);
                        var index = areas.indexOf(scene - 5);

                        if (index !== -1) {
                            areas.splice(index, 1);
                        }
                        record.set('areas', areas);
                        record.commit();
                    }

                }
                dSS.MessageBus.fireEvent('reloaddevicescenevalue');
            });
    },

    getTransmissionQuality: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: can not request transmission quality from a non present device!"));
            return;
        }

        var name = record.get('name');
        var dSUID = record.get('dsuid');
        var DisplayID = record.get('DisplayID');

        Ext.Ajax.request({
            scope: this,
            url: '/json/device/getTransmissionQuality',
            disableCaching: true,
            method: "GET",
            params: {
                dsuid: dSUID
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = dSS._("Transmission Quality");

                        var up = jsonData.result.upstream * (100/dSS.config.TransmissionQuality.UPSTREAM_MAX);
                        var down = 100 - jsonData.result.downstream * (100/dSS.config.TransmissionQuality.DOWNSTREAM_MAX);
                        up = Math.round(up);
                        down = Math.round(down);

                        var win = new dSS.util.DefaultDialogWindow({
                            title: title,
                            dsid: record.get('dsid'),
                            name: name,
                            DisplayID: DisplayID,
                            dsuid: dSUID,
                            extraItems:
                            [
                                {
                                    xtype: 'dssutildialogcontentpanel',
                                    strings:
                                    [
                                        dSS._("Upstream communication")+':&nbsp;',
                                        up + '%',
                                        dSS._("Downstream communication")+':&nbsp;',
                                        down + '%'
                                    ]
                                }
                            ],
                            maskElement: this.findParentByType(Ext.Viewport).el
                        });
                        win.show();

                    } else {
                        Ext.MessageBox.alert(dSS._("Error"),
                                dSS._("Could not get transmission quality: "),
                                  jsonData.message);
                    }
                }
                catch(err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not get transmission quality") +
                                  ': ' + err);
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if ((!jsonData.ok) && (jsonData.message.length > 0)) {
                        message = ': ' + jsonData.message;
                    }
                }
                catch(err) {}
                Ext.MessageBox.alert(dSS._("Error"),
                        dSS._("Could not get transmission quality") + message);
            }
        });
    },

    getDevicePresence: function(item, event, rowIndex) {
        var record;
        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }
            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        var p = new dSS.devprops.SinglePresence(
        {
            width: 400,
            name: record.get('name'),
            displayid: record.get('DisplayID'),
            dsid: record.get('dsid'),
            dsuid: record.get('dsuid'),
            dsmdsuid: record.get('meterDSUID'),
            wasPresent: record.get('isPresent'),
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        p.show();
    },

    removeDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteDeviceConfirm(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Present devices can not be removed!"));
            }
        }
    },

    deleteDeviceConfirm: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.findParentByType(Ext.Viewport).el.unmask();
            return;
        }

        var record = records.shift();

        var removalWarning = '';
        var dSMStore = dSS.data.Loader.getModulatorStore();
        var i = dSMStore.find('dsuid', record.get('meterDSUID'));
        if (i > -1) {
            var dsm = dSMStore.getAt(i);
            var t = dsm.get('busMemberType');
            if (t == dSS.config.BusMemberTypes.dSM11 ||
                t == dSS.config.BusMemberTypes.dSM12 ||
                t == dSS.config.BusMemberTypes.dSM20 ||
                t == dSS.config.BusMemberTypes.dSM25)
            {
                removalWarning = "<p>" + dSS._("Deleting the device will remove its " +
                                "configuration. Therefore, the device should " +
                                "only be deleted if it is permanently " +
                                "removed from this digitalSTROM " +
                                "installation. If you want to connect the " +
                                "device again, you might need to reregister " +
                                "the device manually within the power " +
                                "circuits tab.") + "</p>";
            }
        }

        var win = new dSS.util.DefaultDialogWindow({
            title: dSS._("Remove device"),
            name: record.get('name'),
            DisplayID: record.get('DisplayID'),
            dSUID: record.get('dsuid'),
            cancelButtonText: dSS._("Cancel"),
            scope: this,
            extraItems:
            [
                {
                    xtype: 'panel',
                    html: '<p>' + dSS._("Are you sure you want to delete this device?") + '</p>' + removalWarning
                }
            ],
            maskElement: this.findParentByType(Ext.Viewport).el,
            buttons:
            [
                {
                    xtype: 'button',
                    text: dSS._("Delete"),
                    scope: this,
                    handler: function() {
                        win.close();
                        var f = this.deleteDevice.createDelegate(this,
                                                                 [
                                                                     record,
                                                                     records
                                                                 ]);
                        f();
                    }
                }
            ]
        });
        win.show();
    },

    deleteDevice: function(record, remainingRecords) {
        var maskel = this.findParentByType(Ext.Viewport).el;
        maskel.mask(dSS._("Deleting device") + ": " + record.get('dsuid'));
        Ext.Ajax.request({
            url: '/json/structure/removeDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsuid: record.get('dsuid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var reply = jsonData.result;
                        var dSMStore = dSS.data.Loader.getModulatorStore();

                        if (reply.action === "remove") {
                            for (var i = 0; i < reply.devices.length; i++) {
                                var m = dSMStore.find('dsuid', reply.devices[i].meterDSUID);
                                if (m > -1) {
                                    var dsm = dSMStore.getAt(m);
                                    dsm.set('numDevices', dsm.get('numDevices') - 1);
                                    dsm.commit();
                                }
                                var index = this.getStore().find('dsuid', reply.devices[i].dSUID);
                                if (index > -1) {
                                    this.getStore().removeAt(index);
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                                dSS.MessageBus.fireEvent('statusmessage',
                                        dSS.gt.strargs(dSS._('Device "%1" (%2) was removed'),
                                                [dSS.util.getName(reply.devices[i].name),
                                                reply.devices[i].DisplayID]));

                                dSS.data.Loader.getZoneStore().fireEvent(
                                        'zonedeviceleft', reply.devices[i].zoneID);
                            }
                        }
                        dSS.MessageBus.fireEvent('scenesmarkdirty');
                        this.fireEvent('formsubmitted');

                        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
                            this.updateNumDevicesHeader(undefined);
                        }
                        this.deleteDeviceConfirm(remainingRecords);
                    } else {
                        maskel.unmask();
                        Ext.MessageBox.alert(dSS._("Could not remove"),
                                             dSS._(jsonData.message));
                    }
                }
                catch(err) {
                    maskel.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send remove device command to dSS"));
            }
        });
    },
    blinkDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();
        var i;
        for (i = 0; i < records.length; i++) {
            if (records[i].get('isPresent') === false) {
                continue;
            }

            dSS.util.blinkDevice(records[i].get('dsuid'),records[i].get('name'),
                                 records[i].get('DisplayID'));
        }
    },
    showShadePropertiesWindow: function(item, event, rowIndex) {
        var record;

        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: false,
            width: dSS.gthelper.getDialogConfig('shadePropertyWindowWidth'),
            dialogType: dSS.config.dialogTypes.SHADE_PROPERTIES,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    showHeatingPropertiesWindow: function(item, event, rowIndex) {
        var record;

        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var devInfo = dSS.util.parseProductID(sel.getSelected().get('productID'));
        var devType = devInfo[0];
        var devNumber = devInfo[1];

        var outputMode = record.get('outputMode');

        if (!dSS.util.isValidHeatingOutputMode(outputMode) ||
			(!((devType === dSS.config.deviceTypeNum.ZWS) &&
			 (devNumber === 205) && (dSS.util.getHeatingGroup(record.get('groups')) === dSS.config.groups.TEMPCONTROL)))) {
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: false,
            width: dSS.gthelper.getDialogConfig('heatingPropertyWindowWidth'),
            dialogType: dSS.config.dialogTypes.HEATING_PROPERTIES,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    showDevicePropertiesWindow: function(item, event, rowIndex) {
        var record;
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var white = false;
        var fid = dSS.util.parseFunctionID(record.get('functionID'));
        if ((fid[0] == dSS.config.deviceClass.WE) &&
            (record.get('isVdcDevice') === true)) {
            white = true;
        }

        var jumpMode = false;
        var width = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        if (this.featureConfig === dSS.config.FeatureConfig.SCENE) {
            jumpMode = true;
            if (white === true) {
                try {
                    // will throw if no scene is selected
                    this.findParentByType('dssscenespanel').getSelectedScene();
                    // open appropriate link for apartment scene selection mode
                    // and open the same link for room scenes selection mode
                    dSS.util.openVdcLink(record, 'activities');
                } catch (err) {
                    // no scene selected, for white it means: show customactions
                    dSS.util.openVdcLink(record, 'customactions');
                }
                return;
            }
        } else {
            // for the scenes tab these are the "scene " properties,
            // for all other tabs these are the regular device properties
            if (white === true) {
                // open remote link for "regular" properties of this vdc
                dSS.util.openVdcLink(record, 'settings');
                return;
            }
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: jumpMode,
            featureConfig: this.featureConfig,
            callerScope: this,
            width: width,
            dialogType: dSS.config.dialogTypes.DEVICE_PROPERTIES,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    showDevicePropertiesWindow2: function(item, event, rowIndex) {
        var record;
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var fid = dSS.util.parseFunctionID(record.get('functionID'));
        if ((fid[0] == dSS.config.deviceClass.WE) &&
            (record.get('isVdcDevice') === true)) {
            // open vdc settings (link in the cloud)
            dSS.util.openVdcLink(record, 'settings');
            return;
        }
        var jumpMode = false;

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: jumpMode,
            featureConfig: dSS.config.FeatureConfig.CIRCUIT,
            callerScope: this,
            dialogType: dSS.config.dialogTypes.DEVICE_PROPERTIES,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            ignore: [ 'functionID' ],
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 480,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    editComment: function(item, event, rowIndex) {
        var record;

        if (rowIndex === undefined) {
            // we were called from the toolbar, so its possible that there is
            // no device selection yet
            var sel = this.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("No device selected, please select a device from the list!"));
                return;
            }

            record = sel.getSelected();
        } else {
            record = this.getStore().getAt(rowIndex);
        }

        Ext.Msg.prompt(
            dSS._("Edit comment"),
            dSS._("Comment for") + ' ' + record.get('DisplayID') + ': ',
            function(btn, text) {
            if ((btn === 'ok') && (text !== record.get('comment'))) {
                Ext.Ajax.request({
                    url: '/json/property/setString',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        path:   '/apartment/zones/zone0/devices/' +
                                record.get('dsuid') + '/comment',
                        value: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        result.responseText);
                            if (jsonData.ok) {
                                Ext.Ajax.request({
                                    url: '/json/property/setFlag',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        flag: 'ARCHIVE',
                                        path: '/apartment/zones/zone0/devices/'+
                                                record.get('dsuid') + '/comment',
                                        value: true
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(
                                                result.responseText);
                                            if (jsonData.ok) {
                                                record.set('comment', text);
                                                record.commit();
                                            }
                                        } catch(err) {
                                            Ext.MessageBox.alert(dSS._("Error"),
                                            dSS._("Could not save comment on dSS") +
                                              ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(dSS._("Error"),
                                            dSS._("Could not save comment on dSS"));
                                    }
                                });
                            } else {
                                Ext.MessageBox.alert(dSS._("Error"),
                                                    dSS._("Could not add comment"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(dSS._("Error"),
                                     dSS._("Could not add comment") + ": " + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(dSS._("Error"),
                                dSS._("Could not send command to dSS"));
                    }
                });
            }
        },
        this, true, record.get('comment'));
    },
    showConsumptionThresholdWindow: function(item, event, rowIndex) {
        var record;

        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(record);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (matrix.consumption !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Consumption thresholds can only be configured for klemme L and ZWS devices"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: false,
            width: 554,
            title: dSS._("Configure consumption event"),
            dialogType: dSS.config.dialogTypes.CONSUMPTION_SETTINGS,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },

    reconfigureColumns: function(mode) {
        var col = this.getColumnModel();
        var i;
        var hide;
        if (mode === 'zone') {
            hide = true;
        } else if (mode === 'apartment') {
            hide = false;
        }

        i = col.findColumnIndex('buttonMode');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('meterName');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('zoneName');
        if (i !== -1) {
            col.setHidden(i, hide);
        }

        i = col.findColumnIndex('comment');
        if (i !== -1) {
            col.setHidden(i, hide);
        }
    },

    removeDeviceFromGroup: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        if ((this.selectedGroup === undefined) ||
            (!dSS.util.isUserGroup(this.selectedGroup.get('id')))) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("Can not remove device from group, invalid group selected!"));
            return;
        }
        records = sel.getSelections();

        var skippedNotPresent = false;
        var present = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === false) {
                skippedNotPresent = true;
                continue;
            }

            present.push(record);
        }

        if (present.length > 0) {
            this.removePresentFromGroup(present, this.selectedGroup.get('id'));
        } else {
            if (skippedNotPresent === true) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Present devices can not be removed!"));
            }
        }
    },
    removePresentFromGroup: function(remainingRecords, groupID) {
        var record = remainingRecords.shift();

        var maskel = this.findParentByType(Ext.Viewport).el;
        maskel.mask(
            dSS.gt.strargs(
                dSS._("Removing device %1 from group %2"),
                [
                    '&quot;' +
                    dSS.util.getName(record.get("name")) +
                    '&quot; (' +
                    record.get('DisplayID') +
                    ')',
                    '&quot;' + groupID +  '&quot;'
                ]
            )
        );
        Ext.Ajax.request({
            url: '/json/structure/clusterRemoveDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsuid: record.get('dsuid'),
                clusterID: groupID
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        dSS.util.processDeviceActionResponse(jsonData.result);
                        if (remainingRecords.length > 0) {
                            this.removePresentFromGroup(remainingRecords,
                                                        groupID);
                        } else {
                            maskel.unmask();
                            this.fireEvent('removedfromgroup');
                        }
                    } else {
                        maskel.unmask();
                        this.fireEvent('removedfromgroup');
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not remove device from group"),
                            jsonData.message);
                    }
                }
                catch(err) {
                    maskel.unmask();
                    this.fireEvent('removedfromgroup');
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not send remove device from group command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                this.fireEvent('removedfromgroup');
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send remove device from group command to dSS"));
            }
        });
    },
    unlockDevices: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var locked = [];

        var i;
        for (i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if ((record.get('locked') === true) &&
                (record.get('isPresent') == true)) {
                locked.push(record);
            }
        }

        if (locked.length > 0) {
            this.unlockDevice(locked);
        }
    },
    unlockDevice: function(remainingRecords) {
        var maskel = this.findParentByType(Ext.Viewport).el;

        if (remainingRecords.length < 1) {
            maskel.unmask();
            return;
        }

        var record = remainingRecords.shift();
        maskel.mask(dSS._("Unlocking device") + ": " + record.get('DisplayID'));
        Ext.Ajax.request({
            url: '/json/device/unlock',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsuid: record.get('dsuid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        record.set('locked', false);
                        record.commit();
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Device") + ' &quot;' +
                            dSS.util.getName(record.get('name')) + '&quot; (' +
                            record.get('DisplayID') +
                            ') ' + dSS._("was unlocked"));

                        var paired = record.get('pairedDevices');
                        if (paired.length > 0) {
                            var i;
                            for (i = 0; i < paired.length; i++) {
                                var r = this.getStore().getById(paired[i]);
                                if (r != undefined) {
                                    r.set('locked', false);
                                    r.commit();
                                }
                            }
                        }
                        this.unlockDevice(remainingRecords);
                    } else {
                        maskel.unmask();
                        Ext.MessageBox.alert(dSS._("Could not unlock"), jsonData.message);
                    }
                }
                catch(err) {
                    maskel.unmask();
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send unlock device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                maskel.unmask();
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send unlock device command to dSS"));
            }
        });
    },
    filterDevices: function(mixed) {
        if (!this.isVisible()) {
            return;
        }

        var store = this.getStore();

        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            var zone = mixed;

            // no zone or all zones
            if ((zone === undefined) || (zone.get('zoneId') === 0)) {
                store.clearFilter();
                var title = dSS._("Devices in all rooms");
                this.updateNumDevicesHeader(title);
                this.setInitialSelection();
                return;
            }

            var zoneId = zone.get('zoneId');
            var floorId = zone.get('floorId');

            if (zone.get('isFloor') === true) {
                // show devices from all zones on this floor
                store.filterBy(function(record) {
                    var deviceZone = dSS.util.getZoneById(record.get('zoneId'));
                    if (deviceZone === undefined) {
                        return false;
                    }

                    if (deviceZone.get('floorId') === floorId) {
                        return true;
                    }
                    return false;
                }, this);
            } else {
                // show only devices from this zone
                store.filterBy(function(record) {
                    if (record.get('zoneId') === zoneId) {
                        return true;
                    }
                    return false;
                }, this);
            }
            var title;
            if (zone.get('isFloor') === true) {
                title = dSS._("Devices on floor") + " \"" + zone.get('name') +
                        " - " + floorId + "\"";;
            } else {
                if (zoneId === dSS.config.zones.NEW_DEVICES) {
                    title = dSS._("New devices");
                }
                else {
                    title =  dSS._("Devices in room") + " \"" + zone.get('name') +
                    " - " + zoneId + "\"";
                }
            }
            this.updateNumDevicesHeader(title);
        } else if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
            var zone = mixed;

            dSS.MessageBus.fireEvent('clearsensordata');
            dSS.MessageBus.fireEvent('clearbinaryinputdata');

            var title;
            if ((zone === undefined) || (zone.get('zoneId') === 0)) {
                title = dSS._("Devices in all rooms");
            } else if (zone.get('zoneId') === dSS.config.zones.NEW_DEVICES) {
                title = dSS._("New devices");
            } else {
                title =  dSS._("Devices in room") + " \"" + zone.get('name') +
                 " - #" + zone.get('zoneId') + "\"";
            }

            store.filterBy(function(record) {
                var i;

                if (record.get('isPresent') == false) {
                    return false;
                }

                if ((record.get('sensorInputCount') === 0) &&
                    (record.get('binaryInputCount') === 0)) {
                    return false;
                }

                if ((zone === undefined) || (zone.get('zoneId') === 0)) {
                    return true;
                } else {
                    if (record.get('zoneId') === zone.get('zoneId')) {
                        return true;
                    }
                }

                return false;
            }, this);

            this.updateNumDevicesHeader(title);
        } else if (this.featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
            var dsm = mixed;

            if (dsm === undefined) {
                return;
            }

            var title = dSS._("Device", "Devices", 2);

            store.filterBy(function(record) {
                if (record.get('meterDSUID') === dsm.get('dsuid')) {
                    return true;
                }
                return false;
            }, this);

            this.updateNumDevicesHeader(title);
        } else if (this.featureConfig === dSS.config.FeatureConfig.GROUP) {
            var group = mixed;

            if ((group === undefined) || (group.get('id') === 0)) {
                var title = dSS._("Devices in all groups");

                store.filterBy(function(record) {
                    if (record.get('groups').indexOf(
                            dSS.config.groups.TEMPCONTROL) !== -1) {
                        return false;
                    }

                    return true;
                }, this);
                this.updateNumDevicesHeader(title);
                this.setInitialSelection();
                return;
            }

            var groupId = group.get('id');

            // show only devices from this group
            store.filterBy(function(record) {
                // #14122
                // function id "white" is a special case, the white devices
                // are in the joker group, but need to be displayed as a
                // "single device group"
                var fid = dSS.util.parseFunctionID(record.get('functionID'));
                if ((groupId == dSS.config.groups.SINGLEDEVICES) &&
                    (fid[0] == dSS.config.deviceClass.WE) &&
                    (record.get('groups').indexOf(
                        dSS.config.groups.JOKER) !== -1)) {
                    return true;
                } else {
                    if (record.get('groups').indexOf(groupId) !== -1) {
                        return true;
                    }
                }

                return false;
            }, this);

            var title =  dSS._("Devices in group") + " \"" + group.get('name');
            if (groupId != dSS.config.groups.SINGLEDEVICES) {
                title = title + " - #" + groupId;
            }
            title = title + "\"";

            this.updateNumDevicesHeader(title);
        }
        this.setInitialSelection();
    },
    updateNumDevicesHeader: function(title) {
        var count = this.getStore().getRange().length;
        if (title != undefined) {
            this.lastTitle = title;
        }
        if (title == undefined) {
            if (this.lastTitle == undefined) {
                title = "";
            } else {
                title = this.lastTitle;
            }
        }
        var text = "<span class=\"x-panel-header-text\">" + title +
            "</span><span style=\"float: right;\" class=\"x-panel-header-text\">" +
            count + " " + dSS._("Device", "Devices", count) + "</span>";
        this.setTitle(text);
    },
    showSensorValues: function(item, event, rowIndex) {
        var record;

        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("No device selected, please select a device from the list!"));
            return;
        }

        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Device not present: the selected device was not found and can not be configured!"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: sel,
            jumpMode: false,
            hideSaveButton: true,
            width: 554,
            title: dSS._("Display sensor values"),
            dialogType: dSS.config.dialogTypes.SENSOR_VALUES,
            featureConfig: this.featureConfig,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    checkDisableSVButton: function(rec) {
        var disableSVButton = true;
        if (rec.get('sensorInputCount') > 0) {
            var sensors = rec.get('sensors');
            var i;
            for (i = 0; i < sensors.length; i++) {
                if (dSS.util.isVisibleSensorType(sensors[i].type, true)) {
                    disableSVButton = false;
                    break;
                }
            }
        }

        return disableSVButton;
    }
});

Ext.reg('dssdevicepanel', dSS.grid.DevicePanel);

