Ext.namespace('dSS.util', 'dSS.util.IdentifyDevice');

ButtonKeyType =
{
    "ROOM_BUTTON_1_WAY"         : 0,
    "ROOM_BUTTON_2_WAY_DOWN"    : 1,
    "ROOM_BUTTON_2_WAY_UP"      : 2,
    "LOCAL_BUTTON_1_WAY"        : 4,
    "LOCAL_BUTTON_2_WAY_DOWN"   : 5,
    "LOCAL_BUTTON_2_WAY_UP"     : 6
};

Ext.define('dSS.util.IdentifyDevice.Window', {
    extend: Ext.Window,
    width: 300,
    height: 400,
    layout: 'fit',
    
    constructor:  function(config) {
        
        this.devstore = config.devstore;
        this.dsmstore = config.dsmstore;

        if (config.width === undefined)
        {
            config.width = 413;
        }

        this.maskElement = config.maskElement;

        this.closeButton =
        {
            xtype: 'button',
            text: dSS._("Close"),
            scope: this,
            minWidth: 85,
            handler: function()
            {
                this.close();

                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            }
        };
        
        var hideSave = true;
        if (dSS.module === true)
        {
            hideSave = false;
        }

        this.saveButton = new Ext.Button(
        {
            text: dSS._("Save"),
            disabled: true,
            minWidth: 85,
            hidden: hideSave,
            scope: this,
            handler: this.saveName.createDelegate(this)
        });
        
        this.oldName = "";
        if (dSS.module === true)
        {
            this.deviceName = new Ext.form.TextField(
            {
                fieldLabel: dSS._("Name"),
                disabled: true,
                width: 181,
                scope: this,
                enableKeyEvents: true,
                listeners:
                {
                    keyup: this.handleKeypress.createDelegate(this),
                    specialkey: this.handleSpecialKeypress.createDelegate(this)
                }
            });
        }
        else
        {
            this.deviceName = new Ext.form.Label(
            {
                fieldLabel: dSS._("Name")
            });
        }
        
        Ext.apply(this,
                {
                    title: dSS._("Identify device"),
                    bodyBorder: false,
                    resizable: false,
                    iconCls: 'dSS_IdentifyDeviceTitle',
                    items:
                    [
                        {
                            xtype: 'panel',
                            padding: 9,
                            border: false,
                            bodyBorder: false,
                            frame: false,
                            layout: {
                                type: 'vbox',
                                align: 'stretch',
                                borderPadding: 5
                            },
                            ref: 'main',
                            items:
                            [
                                {
                                    xtype: 'label',
                                    flex: 1,
                                    html: dSS._("Please press a pushbutton to show" +
                                                " device properties.")
                                },
                                {
                                    xtype: 'panel',
                                    layout: 'table',
                                    id: 'info',
                                    width: '100%',
                                    autoScroll: true,
                                    defaults: {
                                        bodyStyle:'padding-left:7px;padding-right:7px;padding-bottom:3px;',
                                        border: false
                                    },
                                    layoutConfig: {
                                        columns: 2
                                    },
                                    flex: 6
                                },
                                {
                                    xtype: 'panel',
                                    layout: 'absolute',
                                    height: 35,
                                    width: 400,
                                    bodyBorder: false,
                                    items:
                                    [
                                        {
                                            xtype: 'panel',
                                            layout: 'form',
                                            bodyBorder: false,
                                            x: 0,
                                            y: 10,
                                            items:
                                            [
                                                this.deviceName
                                            ]
                                        },
                                        {
                                            xtype: 'panel',
                                            x: 296,
                                            y: 10,
                                            bodyBorder: false,
                                            width: 90,
                                            height: 25,
                                            items:
                                            [
                                                this.saveButton
                                            ]
                                        }
                                    ]
                                },
                                {
                                    xtype: 'checkbox',
                                    id: 'filter',
                                    boxLabel: dSS._("Filter for selected dSM/controller only"),
                                    checked: config.isVDC
                                }
                            ]
                        }
                    ],
                    buttonAlign: 'right',
                    buttons:
                    [
                        this.closeButton
                    ]
                });
        
        this.callParent(arguments);
        
        this.on('afterrender',
            function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.mask();
                }
            },
            this);
        
        this.on('beforedestroy',
            function()
            {
                this.closed = true;
                this.webSocket.close();
                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            },
            this);
    },
    checkResults: function(success, data)
    {
        if (this.closed === true)
        {
            return;
        }

        if (data === undefined)
        {
            return;
        }

        if (success !== true)
        {
            this.close();
            return;
        }
        
        var parsed = JSON.parse(data.description);
        if (parseInt('0x' + parsed.dSUID) !== 0)
        {
            this.dsuid = parsed.dSUID;
            this.updateResults(true, parsed);
        }
        else
        {
            this.dsuid = "";
            this.oldName = "";
            if (dSS.module === true)
            {
                this.deviceName.setValue("");
                this.deviceName.disable();
                this.saveButton.disable();
            }
            else
            {
                this.deviceName.setText("");
            }
        }
    },
    updateResults: function(success, parsed)
    {
        if (this.closed === true)
        {
            return;
        }
        var shouldUpdate = (!Ext.getCmp('filter').checked || (parsed.dSMID == this.dsmRecord.get('dsuid')));
        
        if ((success === true) && (shouldUpdate === true))
        {
            delete parsed["dSMID"];//Needed to identify dsm only. Not shown in dialog.
            var rec = this.devstore.getById(parsed.dSUID);
            var name = "";

            if (rec === undefined) {
                var previous = parsed.dSUID.slice(0, -1) + '0';
                rec = this.devstore.getById(previous);
                this.dsuid = previous;
            }
            if (rec !== undefined) {
                name = rec.get('name');
                var zrecord = dSS.util.getZoneById(rec.get('zoneId'));
                if (zrecord !== undefined) {
                    var newRow = Ext.getCmp('info');
                    newRow.removeAll();
                    newRow.add({html: dSS._("Room")}, {html: zrecord.get('name')});
                }
            }
            var unit;
            for(key in parsed) {
                if(key == "sensortype") {
                  unit = dSS.config.SensorTypeUnits[parsed[key]];
                  continue;
                }
                if(unit != undefined) {
                  newRow.add({html: dSS._(key)},{html: parsed[key] + "&nbsp;" + unit});
                  unit = undefined;
                } else {
                  newRow.add({html: dSS._(key)},{html: parsed[key]});
                }
            }
            newRow.doLayout();
            
            this.oldName = name;
            if (dSS.module === true)
            {
                this.deviceName.setValue(name);
                this.deviceName.enable();
            }
            else
            {
                this.deviceName.setText(name);
            }
        }
        else
        {
            this.oldName = "";
            if (dSS.module === true)
            {
                this.deviceName.setValue("");
                this.deviceName.disable();
                this.saveButton.disable();
            }
            else
            {
                this.deviceName.setText("");
            }
        }
    },
    handleKeypress: function(tf, e)
    {
        if (this.deviceName.getValue() !== this.oldName)
        {
            this.saveButton.enable();
        }
        else
        {
            this.saveButton.disable();
        }
    },
    handleSpecialKeypress: function(tf, e)
    {
        if (e.getKey() == e.ENTER)
        {
            this.saveButton.disable();
            if (this.deviceName.getValue() !== this.oldName)
            {
                this.saveName();
            }
        }
    },
    saveName: function()
    {
        this.saveButton.disable();
        if (dSS.module === true)
        {
            var name = this.deviceName.getValue();
            var device = dSS.data.Loader.getDeviceStore().getById(this.dsuid);
            dSS.MessageBus.fireEvent('setdevicename', device, name);
        }
        this.oldName = name;
    }
});
