Ext.namespace("dSS", "dSS.util");

dSS.util.getZoneById = function(zoneId) {
    var store = dSS.data.Loader.getZoneStore();
    var i = store.findExact('zoneId', zoneId);
    if (i < 0) {
        return undefined;
    }
    return store.getAt(i);
};

dSS.util.shortDSUID = function(dSUID) {
    if (dSUID.length > 10) {
        return dSUID.slice(dSUID.length - 10, dSUID.length - 2);
    } else {
        return dSUID;
    }
};

dSS.util.shortToken = function(token) {
    if (token.length > 8) {
        return token.slice(token.length - 8, token.length);
    } else {
        return token;
    }
};

dSS.util.getName = function(name) {
    if ((name === undefined) || (name.length === 0)) {
        return '<i>' + dSS._("no name") + '</i>';
    } else {
        return name;
    }
};

dSS.util.getNameOrDisplayID = function(name, displayID) {
    if (displayID === undefined) {
        displayID = "";
    }

    if ((name === undefined) || (name.length === 0)) {
        return displayID;
    } else {
        return name;
    }
};

dSS.util.decode16 = function(version) {
    var low =  (version & 0x000000ff);
    var high = (version & 0x0000ff00) >> 8;
    return { high: high, low: low };
};

dSS.util.decode16V = function(version) {
    var ret = dSS.util.decode16(version);
    return String(ret.high) + "." + ret.low;
};

dSS.util.decode32 = function(version) {
    var v1 = (version & 0xff000000) >> 24;
    if (v1 < 0) {
        v1 += 256;
    }
    var v2 = (version & 0x00ff0000) >> 16;
    var v3 = (version & 0x0000ff00) >> 8;
    var v4 = (version & 0x000000ff);

    return String(v1) + "." + v2  + "." + v3 + "." + v4;
};

dSS.util.parseFunctionID = function(functionID) {
    var dSclass = (functionID & 0xf000) >> 12;
    var dSsubclass = (functionID & 0xfc0) >> 6;
    var dSfuncModule = (functionID & 0x3f);

    return [dSclass, dSsubclass, dSfuncModule];
};

dSS.util.deviceHasMultipleButtons = function(functionID) {
    var cls = dSS.util.parseFunctionID(functionID);

    if ((cls[1] === 0x04) && ((cls[2] & 0x3) > 1)) {
        return true;
    }
    return false;
};

dSS.util.deviceHasInput = function(functionID) {
    var cls = dSS.util.parseFunctionID(functionID);

    if ((cls[1] === 0x00) && ((cls[2] & 7) != 7)) {
        return true;
    }
    if (((cls[1] === 0x04) || (cls[1] === 0x07)) && ((cls[2] & 0x3) > 0)) {
        return true;
    }
    return false;
};

dSS.util.deviceHasOutput = function(functionID) {
    var rec = dSS.util.parseFunctionID(functionID);

    return ((rec[2] & 0x10) == 0x10) || (functionID === 0x6001);
};

dSS.util.isAreaScene = function(scene) {
    if ((scene < dSS.config.scenes.T1_S0) ||
        (scene > dSS.config.scenes.T4_S1) ||
        (scene == dSS.config.scenes.T0_S1)) {
        return false;
    }
    return true;
};

dSS.util.parseProductID = function(productID) {
    var deviceType = (productID >> 10) & 0x3f;
    var deviceNumber = productID & 0x3ff;
    return [deviceType, deviceNumber];
};

dSS.util.dec2hex = function(d) {
  var hD = '0123456789ABCDEF';
  var h = hD.substr(d & 15, 1);
  while (d > 15) {
    d>>=4;
    h = hD.substr(d & 15, 1) + h;
  }
  return h;
};

dSS.util.callScene = function(dSUID, sceneNumber) {
    Ext.Ajax.request({
        url: '/json/device/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            dsuid: dSUID,
            sceneNumber: sceneNumber,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.callSceneOnZone = function(zoneId, sceneNumber, groupId, force) {
    if (force === undefined) {
        force = false;
    }
    Ext.Ajax.request({
        url: '/json/zone/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId,
            force: force,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.saveSceneOnZone = function(zoneId, sceneNumber, groupId) {
    Ext.Ajax.request({
        url: '/json/zone/saveScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(dSS._("Could not save scene") + " " + sceneNumber, jsonData.message);
                } else {
                    var stmp = dSS._(dSS.config.sceneNames[sceneNumber]);

                    var s = dSS.data.Loader.getSceneStore();
                    var srec = s.getById(zoneId + '-' + groupId + '-' +
                                         sceneNumber);
                    if (srec !== undefined) {
                        stmp = dSS.util.getName(srec.get('name'));
                    }

                    var msg = dSS.gt.strargs(dSS._('Scene "%1" in zone #%2 was saved'),
                            [stmp, zoneId]);
                    if (zoneId != 0) {
                        var zrec = dSS.util.getZoneById(zoneId);
                        if (zrec !== undefined) {
                            msg = dSS.gt.strargs(dSS._('Scene "%1" in zone "%2" was saved'),
                                [stmp, dSS.util.getName(zrec.get('name'))]);
                        }
                    }

                    dSS.MessageBus.fireEvent('statusmessage', msg);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS"));
        }
    });
};

dSS.util.setValueOnDevice = function(dSUID, value) {
    Ext.Ajax.request({
        url: '/json/device/setValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsuid: dSUID,
            value: value,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS"));
        }
    });
};

dSS.util.setOutputValueOnDevice = function(dSUID, offset, value) {
    Ext.Ajax.request({
        url: '/json/device/setOutputValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsuid : dSUID,
            offset: offset,
            value: value,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send setOutputValue command to dSS") +
                          ": " + err);
            }
        },
        failure: function(result, request) {
            var msg = '';
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (jsonData.message) {
                    msg = ': ' + dSS._(jsonData.message);
                }
            } catch (err) {
                msg = ': ' + err;
            }
            Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not send setOutputValue command to dSS") +
                        msg);
        }
    });
};

dSS.util.setSceneValue = function(dSUID, scene, value, angle) {
    Ext.Ajax.request({
        url: '/json/device/setSceneValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsuid : dSUID,
            sceneID: scene,
            value: value,
            angle: angle
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send setSceneValue command to dSS") +
                          ": " + err);
            }
        },
        failure: function(result, request) {
            var msg = '';
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (jsonData.message) {
                    msg = ': ' + dSS._(jsonData.message);
                }
            } catch (err) {
                msg = ': ' + err;
            }
            Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not send setSceneValue command to dSS") +
                        msg);
        }
    });
};

/* parameter object
    {
        url, // url string
        params // ajax request parameters
        failureMsg // failure message to show in alert messagebox
        maskEl // element to mask during loading
        maskMsg // mask message
        scope // object scope (passt to ajax requuest)
        evt // event name (string) to fire
    }
    returns AjaxRequest object cotnainig the transaction id
*/
dSS.util.genericRequest = function(param, callback) {
    if (param === undefined) {
        throw dSS._("Call with undefined parameter");
    }

    if (param.maskEl !== undefined) {
        if (param.maskMsg !== undefined) {
            param.maskEl.mask(param.maskMsg + '...', 'x-mask-loading');
        } else {
            param.maskEl.mask();
        }
    }

    return Ext.Ajax.request(
    {
        url: param.url,
        method: 'GET',
        params: param.params,
        scope: param.scope,
        success: function(response) {
            var ok = false;
            var jsonData;
            try {
                jsonData = Ext.util.JSON.decode(
                                    response.responseText);
                if (jsonData.ok !== true) {
                    var errmsg = '';
                    if (jsonData.message !== undefined) {
                        errmsg = ': ' + jsonData.message;
                    }
                    if (param.failureMsg !== undefined) {
                        if (jsonData.message !== undefined) {
                            errmsg = ': ' + jsonData.message;
                        }
                        Ext.Msg.alert(dSS._("Error"), param.failureMsg +
                                      errmsg);
                    } else {
                        if (jsonData.message !== undefined) {
                            errmsg = jsonData.message;
                        }
                        Ext.Msg.alert(dSS._("Error"), errmsg);
                    }
                } else {
                    ok = true;
                }
            } catch (error) {
                Ext.Msg.alert(dSS._("Error"), error);
            }

            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(ok, jsonData);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt, true); // scope set by ajax request
            }
        },
        failure: function(result, request) {
            var msg;
            try {
                var errmsg = '';
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (param.failureMsg !== undefined) {
                    if (jsonData.message !== undefined) {
                        errmsg = ': ' + jsonData.message;
                    }

                    msg = param.failureMsg + errmsg;
                } else {
                    if (jsonData.message !== undefined) {
                        msg = jsonData.message;
                    }
                }
            } catch (err) {
                if (param.failureMsg !== undefined) {
                    msg = param.failureMsg + ': ' + err;
                } else {
                    msg = err;
                }
            }

            Ext.Msg.alert(dSS._("Error"), msg);
            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(false);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt, false);
            }
        }
    });
};

dSS.util.getJokerGroup = function(groups) {
    var i;
    var group = null;
    var joker = false;

    for (i = 0; i < groups.length; i++) {
        group = groups[i];
        if ((group < dSS.config.groups.JOKER) ||
            (group == dSS.config.groups.APT_VENTILATION) ||
            (group == dSS.config.groups.VENTILATION) ||
            (group == dSS.config.groups.RECIRCULATION) ||
            (group == dSS.config.groups.TEMPCONTROL)) {
            return group;
        } else if (group === dSS.config.groups.JOKER) {
            joker = true;
        }
    }

    if (joker === true) {
        return dSS.config.groups.JOKER;
    }

    return null;
};

dSS.util.getHeatingGroup = function(groups) {
    var i;
    var group = null;
    var joker = false;

    for (i = 0; i < groups.length; i++) {
        group = groups[i];
        if ((group === dSS.config.groups.HEATING) ||
             (group === dSS.config.groups.COOLING) ||
             (group === dSS.config.groups.VENTILATION) ||
             (group === dSS.config.groups.RECIRCULATION) ||
             (group === dSS.config.groups.WINDOW) ||
             (group === dSS.config.groups.TEMPCONTROL)) {
            return group;
        }
    }

    return null;
};

dSS.util.updateJokerGroup = function(groups, oldGroup, newGroup) {
    var i;
    for (i = 0; i < groups.length; i++) {
        if (oldGroup === groups[i]) {
            groups[i] = newGroup;
        }
    }
    return groups;
};

dSS.util.UniqueArray = function(array, ints) {
  var i;
  var length = array.length;
  var helper = {};
  var result = [];

  for (i = 0; i < length; i++) {
      helper[array[i]] = 0;
  }

  for (i in helper) {
      if (ints === true) {
          result.push(parseInt(i, 10));
      } else {
          result.push(i);
      }
  }

  return result;
};

dSS.util.ResetDirtyField = function(field) {
    field.originalValue = field.getRawValue();
};

dSS.util.PushButtonToString = function(buttonID, jokerFlag) {
    var dec = (buttonID & 0x0000000f);
    ret = "";

    switch (dec)
    {
        case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
            ret = dSS._("Device");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
        case dSS.config.deviceParams.PUSHBUTTON_ALARM:
            if (jokerFlag === true) {
                ret = dSS._("alarm");
                break;
            }
            ret = dSS._("Area 1") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
            ret = dSS._("Area 1") + ", " + dSS._("Scenes 12-14");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
        case dSS.config.deviceParams.PUSHBUTTON_PANIC:
            if (jokerFlag === true) {
                ret = dSS._("Panic");
                break;
            }
            ret = dSS._("Area 2") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
            ret = dSS._("Area 2") + ", " + dSS._("Scenes 22-24");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
        case dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME:
            if (jokerFlag === true) {
                ret = dSS._("Leaving home");
                break;
            }
            ret = dSS._("Area 3") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
            ret = dSS._("Area 3") + ", " + dSS._("Scenes 32-34");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            ret = dSS._("Area 4") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
            ret = dSS._("Area 4") + ", " + dSS._("Scenes 42-44");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
        case dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL:
            if (jokerFlag === true) {
                ret = dSS._("Door bell");
                break;
            }
            ret = dSS._("Scenes 0-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            ret = dSS._("Scenes 10-14");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            ret = dSS._("Scenes 20-24");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            ret = dSS._("Scenes 30-34");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
            ret = dSS._("Scenes 40-44");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
            ret = dSS._("server button");
            break;
        default:
            break;
    }

    return ret;
};

dSS.util.PushButtonModeToString = function(buttonMode, buttonIndex, buttonCount,
                                           group, isSDS) {
    var dec = (buttonMode & 0x000000ff);
    var ret = "";
    var includeLine = true;

    switch (dec)
    {
        default:
        case dSS.config.deviceParams.BUTTONINPUT_STANDARD:
        case dSS.config.deviceParams.BUTTONINPUT_TURBO:
            break;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_STANDARD:
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_EDGE:
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_TIP:
            ret = dSS._("Switch");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT2:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT3:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4:
            if (group === dSS.config.groups.SHADOW) {
                ret = dSS._("2-way up");
            } else {
                ret = dSS._("2-way on");
            }
            includeLine = false;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT2:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT3:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4:
            if (group === dSS.config.groups.SHADOW) {
                ret = dSS._("2-way down");
            } else {
                ret = dSS._("2-way off");
            }
            includeLine = false;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY:
            ret = dSS._("2-way");
            includeLine = false;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_1WAY:
            ret = dSS._("1-way");
            includeLine = false;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_STANDARD:
            ret = dSS._("State normal");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_INVERTED:
            ret = dSS._("State inv");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_ON_RISINGEDGE:
            ret = dSS._("State on/rising");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_ON_FALLINGEDGE:
            ret = dSS._("State on/falling");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_RISINGEDGE:
            ret = dSS._("State off/rising");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_OFF_FALLINGEDGE:
            ret = dSS._("State off/falling");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_RISINGEDGE:
            ret = dSS._("State rising");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_AKM_FALLINGEDGE:
            ret = dSS._("State falling");
            break;
    }

    if(includeLine && (buttonCount > 1)) {
        var lineText = dSS.gt.strargs(dSS._("Line %1"), [buttonIndex + 1]);
        if(isSDS) {
            lineText = dSS.gt.strargs(dSS._("Button %1"), [buttonIndex + 1]);
        }
        if(ret !==  "") {
            ret = lineText + ", " + ret;
        } else {
            ret = lineText;
        }
    }

    return ret;
};

dSS.util.AKMModeToString = function(inputProperty, buttonIndex, buttonCount) {
    var ret = "";

    ret = dSS._(dSS.config.AKMButtonInputStrings[inputProperty]);

    if((buttonCount > 1)) {
        var lineText = dSS.gt.strargs(dSS._("Line %1"), [buttonIndex + 1]);
        if(ret !==  "") {
            ret = lineText + ", " + ret;
        } else {
            ret = lineText;
        }
    }

    return ret;
};

// "ok" paramter is needed in order to be able to chain the calls
dSS.util.setConfig = function(ok, dSUID, cls, index, value, cb) {
    if (ok !== true) {
        if (cb !== undefined) {
            cb(ok);
        }
        return;
    }

    dSS.util.genericRequest({
            url: '/json/device/setConfig',
            params:
            {
                'class': cls,
                index: index,
                dsuid : dSUID,
                value: value
            },
            failureMsg: dSS._("Could not write device " +
                              "configuration"),
            scope: this
        },
        cb);
};

dSS.util.getControllerStatuses = function(store) {
    Ext.Ajax.request({
        url: '/json/apartment/getControllerStatuses',
        disableCaching: true,
        method: "GET",
        params: {
            lang: dSS.config.language
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Failed getting controller statuses"));
                } else {
                	dSS.util.updateControllerStatuses(store, jsonData.result.controllerStatuses);
                }
            }
            catch(err) {
                dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Failed getting controller statuses") + ": " +
                        err);
            }
        },
        failure: function(result, request) {
            dSS.MessageBus.fireEvent('statusmessage',
                    dSS._("Failed getting controller statuses"));
        }
    });
},

dSS.util.updateControllerStatuses = function(store, controllers) {
    store.each(
        function(record) {
            var code = "off";
            var message = dSS._("Status not available");
            for (var i in controllers) {
                if (controllers.hasOwnProperty(i)) {
                    if (record['id'] == i) {
                        switch(controllers[i].code) {
                        case "ready":
                            code = "ready";
                            message = dSS._("Ready");
                            break;
                        case "error":
                            code = "error";
                            message = dSS._("Error");
                            break;
                        case "off":
                        default:
                            code = "off";
                            message = dSS._("Off");
                        }
                        if (controllers[i].message) {
                            message = controllers[i].message;
                        }
                        break;
                    }
                }
            }
            record.set('ledCode', code);
            record.set('ledStatus', message);
        } 
    );
    store.commitChanges();
};

dSS.util.setCSS = function(css) {
    var head = document.getElementsByTagName("head");
    if (head.length > 0) {
        head = head[0];
    }

    if (head.children.length > 0) {
        for (var i = 0; i < head.children.length; i++) {
            var style = head.children[i];
            if (style.nodeName !== "STYLE") {
                continue;
            }
            if (style.id && style.id === "dSdeviceIcons") {
                head.removeChild(style);
                break;
            }
        }
    }

    var styleEl = document.createElement("style");
    styleEl.setAttribute("id", "dSdeviceIcons");
    styleEl.type = "text/css";
    newElement = true;

    if (styleEl.styleSheet) {
        styleEl.styleSheet.cssText = css;
    } else {
        styleEl.appendChild(document.createTextNode(css));
    }
    head.appendChild(styleEl);
};

dSS.util.genCSSName = function(record) {
    var color = "";
    var cls = dSS.util.parseFunctionID(record.get('functionID'));
    if (cls[0] === dSS.config.deviceClass.SW) {
        color = dSS.util.getJokerGroup(record.get('groups'));
        if (color === null) {
            color = dSS.config.deviceClass.SW;
        }
        color = 'joker' + color;
    }
    return record.get('dsuid') + color;
};

dSS.util.undoSceneOnZone = function(zoneId, sceneNumber) {
    Ext.Ajax.request({
        url: '/json/zone/undoScene',
        disableCaching: true,
        method: "GET",
        params: {
            sceneNumber: sceneNumber,
            id: zoneId,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not call undo scene") +
                                  ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not send undoScene command to dSS") +
                              ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Could not send undoScene command to dSS"));
        }
    });
};

dSS.util.maskViewport = function(operation) {
    if (this.isVisible() !== true) {
        return;
    }

    var maskel = this.findParentByType(Ext.Viewport).el;
    if (maskel !== undefined) {
        if (operation === 'mask') {
            maskel.mask(dSS._("Loading data") + '...', 'x-mask-loading');
        } else {
            maskel.unmask();
        }
    }
};

dSS.util.getDeviceColor = function(deviceRecord) {
    var functionID = deviceRecord.get('functionID');
    var fid = dSS.util.parseFunctionID(functionID);
    var col = fid[0];
    if (col === dSS.config.deviceClass.SW) {
        var jc = dSS.util.getJokerGroup(deviceRecord.get('groups'));
        if (jc !== null) {
            col = jc;
        }
    } else if (col === dSS.config.deviceClass.BL) {
        var hc = dSS.util.getHeatingGroup(deviceRecord.get('groups'));
        if (hc !== null) {
            col = hc;
        }
    }
    return col;
};

dSS.util.setName = function(record, newName, featureConfig, evt) {
    if (record === undefined) {
        return;
    }

    var params =
    {
        dsuid: record.get('dsuid'),
        newName: newName
    };
    var dSUIDText;
    var failureMsg = dSS._("Could not rename device");
    var url = '/json/device/setName';
    var statusMsg = dSS._("Device \"%1\" (%2) was renamed");

    var defaultName = "";
    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        failureMsg = dSS._("Could not rename meter");
        url = '/json/circuit/setName';
        statusMsg = dSS._("Meter \"%1\" (%2) was renamed");
        dSUIDText = record.get('DisplayID');
        params =
        {
            dsuid : record.get('dsuid'),
            newName: newName
        };
    } else if (featureConfig === dSS.config.FeatureConfig.ZONE) {
        if (record.get('isFloor') === true) {
            statusMsg = dSS._("Floor \"%1\" (%2) was renamed");
            failureMsg = dSS._("Could not rename floor");
            url = '/json/structure/setFloorName';
            params =
            {
                floorID: record.get('floorId'),
                name: newName
            };
            dSUIDText = '#' + record.get('floorId');
            if ((newName === undefined) || (newName.length === 0)) {
                newName = dSS.gt.strargs(dSS._("Floor #%1"), [record.get('floorId')]);
            }
        } else {
            statusMsg = dSS._("Room \"%1\" (%2) was renamed");
            failureMsg = dSS._("Could not rename room");
            url = '/json/zone/setName';
            params =
            {
                id: record.get('zoneId'),
                newName: newName
            };
            dSUIDText = '#' + record.get('zoneId');
            if ((newName === undefined) || (newName.length === 0)) {
                newName = dSS.gt.strargs(dSS._("Zone #%1"), [record.get('zoneId')]);
            }
        }
    } else {
        dSUIDText = record.get('DisplayID');
    }

    var oldName = record.get('name');

    dSS.util.genericRequest({
            url: url,
            params: params,
            failureMsg: failureMsg,
            scope: this,
            evt: evt
        },
        function(success) {
            if (success === true) {
                if (featureConfig === dSS.config.FeatureConfig.ZONE) {
               }
                record.set('name', newName);
                record.commit();

                if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
                    var devStore = dSS.data.Loader.getDeviceStore();
                    devStore.each(
                        function(devRecord) {
                            if (devRecord.get('meterDSUID') === record.get('dsuid')) {
                                devRecord.set('meterName', newName);
                                devRecord.commit();
                            }
                        }
                    );
                } else if (featureConfig === dSS.config.FeatureConfig.ZONE) {
                    var devStore = dSS.data.Loader.getDeviceStore();
                    devStore.each(
                        function(devRecord) {
                            if (devRecord.get('isFloor') === true) {
                                if (devRecord.get('floorId') === record.get('floorId')) {
                                    devRecord.set('zoneName', newName);
                                    devRecord.commit();
                                }
                            } else {
                                if (devRecord.get('zoneId') === record.get('zoneId')) {
                                    devRecord.set('zoneName', newName);
                                    devRecord.commit();
                                }
                            }
                        }
                    );
                } else {
                    // we are only interested in name changes
                    // from nothing to something or vice versa
                    //
                    // check that the names were not equal is done
                    // before submitting the ajax request
                    if ((oldName === undefined) ||
                        (oldName.length === 0) ||
                        (newName === undefined) ||
                        (newName.length === 0)) {
                        dSS.data.Loader.getZoneStore().fireEvent(
                                'zonedevnamechanged', record);
                    }
                }
                dSS.MessageBus.fireEvent('statusmessage',
                    dSS.gt.strargs(statusMsg,
                        [
                            dSS.util.getName(newName), dSUIDText
                        ]));

            } // (success === true)
        } // success callback
    );
};

dSS.util.blinkDevice = function(dSUID, name, displayID) {
    Ext.Ajax.request({
        url: '/json/device/blink',
        disableCaching: true,
        method: "GET",
        scope: this,
        params: {
            dsuid: dSUID,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Could not blink"), jsonData.message);
                } else {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Device \"%1\" (%2) is blinking now"),
                            [
                                dSS.util.getName(name),
                                displayID
                            ]));
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send blink device command to dSS") +
                    ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("Could not send blink device command to dSS"));
        }
    });
};

dSS.util.blinkZone = function(id, name) {
    Ext.Ajax.request({
        url: '/json/zone/blink',
        disableCaching: true,
        method: "GET",
        scope: this,
        params: {
            id: id,
            category: "manual"
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS.gt.strargs(
                        dSS._("Could not blink room \"%1\": %2"),
                        [ dSS.util.getName(name), jsonData.message ]));
                } else {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS.gt.strargs(
                            dSS._("Room \"%1\" (%2) is blinking now"),
                            [ dSS.util.getName(name), id ]));
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS.gt.strargs(dSS._("Could not blink room \"%1\": %2"),
                        [ dSS.util.getName(name), err ]));
            }
        },
        failure: function(result, request) {
            Ext.Msg.alert(dSS._("Error"),
                dSS._("Could not send blink room command to dSS"));
        }
    });
};

dSS.util.isUserGroup = function(id) {
    return ((id >= dSS.config.UserGroupStartIndex) &&
            (id <= dSS.config.UserGroupEndIndex));
};

dSS.util.isControlGroup = function(id) {
    return ((id >= dSS.config.ControlGroupStartIndex) &&
            (id <= dSS.config.ControlGroupEndIndex));
};

dSS.util.isAllowedSensorType = function(sensorType) {
    switch (sensorType)
    {
        case dSS.config.SensorTypes.TEMPERATURE_INDOORS:
        case dSS.config.SensorTypes.TEMPERATURE_OUTDOORS:
        case dSS.config.SensorTypes.HUMIDITY_INDOORS:
        case dSS.config.SensorTypes.HUMIDITY_OUTDOORS:
        case dSS.config.SensorTypes.BRIGHTNESS_INDOORS:
        case dSS.config.SensorTypes.BRIGHTNESS_OUTDOORS:
        case dSS.config.SensorTypes.CO2_CONCENTRATION:
        case dSS.config.SensorTypes.CO_CONCENTRATION:
        case dSS.config.SensorTypes.WIND_SPEED:
        case dSS.config.SensorTypes.WIND_DIRECTION:
        case dSS.config.SensorTypes.GUST_SPEED:
        case dSS.config.SensorTypes.GUST_DIRECTION:
        case dSS.config.SensorTypes.PRECIPITATION:
        case dSS.config.SensorTypes.AIR_PRESSURE:
        case dSS.config.SensorTypes.SOUND_PRESSURE:
            return true;
        default:
            return false;
    }
};

dSS.util.isVisibleSensorType = function(sensorType, mode) {
    // mode true == standalone/inside the dialoge
    // mode false == sensor tab
    if (mode !== true) {
        switch (sensorType) {
            case dSS.config.SensorTypes.RESERVED1:
            case dSS.config.SensorTypes.RESERVED2:
            case dSS.config.SensorTypes.RESERVED3:
            case dSS.config.SensorTypes.RESERVED4:
                return false;
            default:
                return true;
        }
    }

    switch (sensorType)
    {
        case dSS.config.SensorTypes.TEMPERATURE_INDOORS:
        case dSS.config.SensorTypes.TEMPERATURE_OUTDOORS:
        case dSS.config.SensorTypes.HUMIDITY_INDOORS:
        case dSS.config.SensorTypes.HUMIDITY_OUTDOORS:
        case dSS.config.SensorTypes.BRIGHTNESS_INDOORS:
        case dSS.config.SensorTypes.BRIGHTNESS_OUTDOORS:
        case dSS.config.SensorTypes.CO2_CONCENTRATION:
        case dSS.config.SensorTypes.CO_CONCENTRATION:
        case dSS.config.SensorTypes.WIND_SPEED:
        case dSS.config.SensorTypes.WIND_DIRECTION:
        case dSS.config.SensorTypes.GUST_SPEED:
        case dSS.config.SensorTypes.GUST_DIRECTION:
        case dSS.config.SensorTypes.PRECIPITATION:
        case dSS.config.SensorTypes.AIR_PRESSURE:
        case dSS.config.SensorTypes.SOUND_PRESSURE:
            return true;
        default:
            return false;
    }
};

dSS.util.isHeatingPWMMode = function(outputMode) {
    if ((outputMode === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_PWM)) {
        return true;
    }
    return false;
};

dSS.util.isValidHeatingOutputMode = function(outputMode) {
    if ((outputMode === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_SWITCH) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_PWM) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS) ||
        (outputMode === dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_PARALLEL)) {
        return true;
    }

    return false;
};

dSS.util.isApartmentScene = function(scene) {
    return (dSS.config.ApartmentScenes.indexOf(scene) !== -1);
};

dSS.util.getWindProtectionString = function(wpc)
{
    try {
        return dSS.config.WindProtectionClassStrings[wpc];
    } catch (err) {
        return "";
    }

    return "";
};

dSS.util.getClusterLocationString = function(loc)
{
    try {
        return dSS.config.ClusterLocationStrings[loc];
    } catch (err) {
        return "";
    }

    return "";
};

dSS.util.SceneToString = function(scene_num)
{
    var name = dSS.config.sceneNames[scene_num];
    if (name !== undefined) {
        return name;
    } else {
        return "";
    }
};

dSS.util.GetSensorIndex = function(device, sensorType)
{
    var sensors = device.get('sensors');
    if (!sensors) {
        return -1;
    }

    var i;
    for (i = 0; i < sensors.length; i++) {
        if (sensors[i].type == sensorType) {
            return i;
        }
    }

    return -1;
};

dSS.util.Sensor2Temperature = function(val) {
    // copied from dSS
    var v = (val * 0.1 / 4) - 273.15 + 230.0;
    return Math.round(v * 10) / 10;
};

dSS.util.openVdcLink = function(record, configTarget) {
    var callback = function(success, data) {
        // if success is false, generic request will show the error
        if (success === true) {
            if ((data.result != undefined) &&
                (data.result.productInfo != undefined) &&
                (data.result.productInfo.length > 0)) {

                var info = data.result.productInfo[0];
                if ((info.State !== "Valid") ||
                    (info.ProductState !== "Valid")) {
                    Ext.Msg.alert(dSS._("Warning"),
                        dSS._("The configuration page location is " +
                              "unknown and cannot be loaded."));
                    return;
                }

                var ean = info.EAN;
                var url = info.ConfigLink;
                url = url + "?dsuid=" + record.get('dsuid') +
                      '&gtin=' + ean + '&lang=' + dSS.config.language +
                      '&token=' + dSS.data.Loader.vdcToken +
                      '&context=' + configTarget;
                if (dSS.data.Loader.Feed.length > 0) {
                    url = url + '&feed=' + dSS.data.Loader.Feed;
                }
                window.open(url);
            } else {
                // the nodes in the proptree should never be missing,
                // so this is just a safeguard if the dSS messes up
                Ext.Msg.alert(dSS._("Warning"),
                    dSS._("The configuration page location is " +
                          "unknown and cannot be loaded."));
            }
        }
    };

    dSS.util.genericRequest({
        url: '/json/property/query',
        params: {
            query: '/apartment/zones/zone0/devices/' + record.get('dsuid') +
                   '/productInfo(EAN,ConfigLink,State,ProductState)'
        },
        failureMsg: dSS._("Configuration page not available. Please try again later.")
    }, callback);
};

dSS.util.processDeviceActionResponse = function(reply, supressEvent) {
	var dS = dSS.data.Loader.getDeviceStore();
    var mS = dSS.data.Loader.getModulatorStore();

    if (reply && reply.action === "add") {
        reply.device.firstSeen = Date.parseDate(
            reply.device.firstSeen, "Y-m-d H:i:s",
            true);
        reply.device.inactiveSince = Date.parseDate(
            reply.device.inactiveSince, "Y-m-d H:i:s",
            true);

        var z = dSS.util.getZoneById(reply.device.zoneID);
        reply.device.zoneName = z.get('name');
        if ((reply.device.name === undefined) ||
            (reply.device.name.length === 0)) {
            z.set('unnamedDevices', true);
            z.commit();
        }

        var dsm = mS.getById(reply.device.meterDSUID);
        if (dsm !== undefined) {
            reply.device.meterName = dsm.get("name");
            dsm.set('numDevices', dsm.get('numDevices') + 1);
        }

        var dev = { devices : [] };
        dev.devices.push(reply.device);
        dS.loadData(dev, true);
    } else if (reply && reply.action === "remove") {
         index = dS.find('dsuid', reply.device.dSUID);
         if (index > -1) {
             dS.removeAt(index);
         }

         var dsm = mS.getById(reply.device.meterDSUID);
         if (dsm !== undefined) {
            dsm.set('numDevices', dsm.get('numDevices') - 1);
         }
         dSS.data.Loader.getZoneStore().fireEvent(
                 'zonedeviceleft', reply.device.zoneID);
    } else if (reply && reply.action === "update") {
        var i;
        var zoneStore = dSS.data.Loader.getZoneStore();
        for (i = 0; i < reply.devices.length; i++) {
            var zone = dSS.util.getZoneById(reply.devices[i].zoneID);
            if (zone !== undefined) {
                reply.devices[i].zoneName = zone.get('name');
            }
            reply.devices[i].firstSeen =
                Date.parseDate(
                   reply.devices[i].firstSeen,
                   "Y-m-d H:i:s", true);
            reply.devices[i].inactiveSince =
                Date.parseDate(
                    reply.devices[i].inactiveSince,
                    "Y-m-d H:i:s", true);
        }
        // parser is set to devices root element
        dS.loadData(reply, true);
    }
    if (supressEvent !== true) {
        dSS.MessageBus.fireEvent('scenesmarkdirty');
    }
};

dSS.util.processDeviceActionResponseV2 = function(reply, supressEvent) {
    var i;
    for (i = 0; i < reply.actions.add.length; i++) {
        dSS.util.processDeviceActionResponse({ "action": "add", "device": reply.actions.add[i] }, true);
    }
    if (reply.actions.update.length > 0) {
        dSS.util.processDeviceActionResponse({ "action": "update", "devices": reply.actions.update }, true);
    }

    for (i = 0; i < reply.actions.remove.length; i++) {
        dSS.util.processDeviceActionResponse({ "action": "remove", "device": { "dSUID": reply.actions.remove[i].dSUID } }, true);
    }
    if (supressEvent !== true) {
        dSS.MessageBus.fireEvent('scenesmarkdirty');
    }
};

dSS.util.getSystemOverviewSortValue = function(what) {
    var ret;

    try {
        ret = dSS.config.SystemOverviewStateSortOder[what];
    } catch(err) {}

    if (ret === undefined) {
        return 100;
    }

    return ret;
};

dSS.util.percentToValue = function(percent, maxValue) {
    var value = percent / 100;
    value = value * maxValue;
    if (value < 0) {
        value = 0;
    } else if (value > maxValue) {
        value = maxValue;
    }
    return Math.round(value);
};

dSS.util.valueToPercent = function(value, maxValue) {
    if (value < 0) {
        value = 0;
    }  else if (value > maxValue) {
        value = maxValue;
    }
    return (value / maxValue * 100);
};

dSS.util.GetLockString = function(grouprecord) {
    var ret = dSS._("No operation lock is enabled.");

    var oplocks = "";

    if (grouprecord.get('windlock') === true) {
        oplocks = dSS._("wind");
    }

    if (grouprecord.get('rainlock') === true) {
        if (oplocks.length > 0) {
            oplocks = oplocks + ', ';
        }
        oplocks = oplocks + dSS._("rain");
    }

    if (grouprecord.get('frostlock') === true) {
        if (oplocks.length > 0) {
            oplocks = oplocks + ', ';
        }
        oplocks = oplocks + dSS._("frost");
    }

    if (grouprecord.get('haillock') === true) {
        if (oplocks.length > 0) {
            oplocks = oplocks + ', ';
        }
        oplocks = oplocks + dSS._("hail");
    }

    if (oplocks.length > 0) {
        ret = dSS._("Operation lock is enabled for") + ': ' + oplocks;
    }

    if (grouprecord.get('locked') === true) {
        if (ret.length > 0) {
            ret = ret + '<br/>';
        }

        ret = ret + dSS._("The group is locked and cannot be modified");
    }

    return ret;
};

dSS.util.gridGetEANString = function(record) {
    var eanString;

    if (record.get('OemStatus') !== 'Valid') {
        return eanString;
    }

    var serial = record.get('OemSerialNumber');
    var eanName = record.get('HW');
    if ((eanName !== undefined) && (eanName.length > 0)) {
        eanString = eanName + '<br/>';
    }
    var ean = record.get('OemEanNumber');
    if ((ean !== undefined) && (ean.length > 0)) {
        eanString += dSS._("GTIN") + ": " + ean + '<br/>';
    }
    eanString += dSS._("Serial number") + ": " +
        record.get('OemSerialNumber');
    return eanString;
};


dSS.util.gridDeviceNameRenderer = function(value, metaData, record, rowIndex,
                                           colIndex, store) {
    var eanString = dSS.util.gridGetEANString(record);
    var name = record.get('name') ? record.get('name') : '&nbsp;&nbsp;&nbsp;&nbsp;';
    if ((eanString !== undefined) && (eanString.length > 0)) {
        return String.format('<div ext:qtip="{0}">{1}</div>',
                             eanString, name);
    } else {
        return value;
    }
};

dSS.util.gridZoneNameRenderer = function(value, metaData, record, rowIndex,
                                         colIndex, store) {
    return value ? value: dSS.gt.strargs(dSS._("Zone #%1"), [record.get('zoneId')]);
};

dSS.util.gridDeviceTypeRenderer = function(value, metaData, record, rowIndex,
                                           colIndex, store) {
    var retval = String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
    var eanString = dSS.util.gridGetEANString(record);
    if ((eanString !== undefined) && (eanString.length > 0)) {
        retval = String.format('<div ext:qtip="{0}">{1}&nbsp;&nbsp;</div>',
                                eanString, retval);
    }

    if (record.get('isValid') === false) {
        metaData.css = "deviceInfoLoading";
        return retval;
    }

    if (record.get('locked') === true) {
        metaData.css = "deviceLocked";
        return retval;
    }

    metaData.css = "gridIcon" + dSS.util.genCSSName(record);
    return retval;
};

dSS.util.gridDeviceDSUIDRenderer = function(value, metaData, record, rowIndex,
                                            colIndex, store) {
    var shortId = record.get('DisplayID');
    if (record.get('isValid') === false) {
        shortId = dSS._('Loading') + ' ... ' + shortId;
    } else {
        if (record.get('sensorDataValid') == false) {
            metaData.attr = 'style="color:red;"';
        }
    }

    var dsid = "";
    if (record.get('dsid') !== undefined) {
        dsid = record.get('dsid');
    }

    if (dsid.length > 0) {
        return String.format('<div ext:qtip="dSID: {0}<br/>dSUID: {1}">{2}</div>', dsid, record.get('dsuid'), shortId);
    } else {
        return String.format('<div ext:qtip="dSUID: {0}">{1}</div>', record.get('dsuid'), shortId);
    }
};
